/* Command-line parser.
 *
 * PegSoft command-line parser library (c) 2004 PegSoft
 * Contact us at pegsoft@pegsoft.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software (the COPYING file); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 */

#include <system/headers.h>

#include <utility>

#include "cmdline/cmdline.h"

using namespace std;

/*************************************************************************/

namespace cmdline
{

/*************************************************************************/

bool CmdLineParser::parse()
{
	char const *end, *slash = strrchr(m_argv[0], '/');
	
	/* Finds program file name. */
	m_progname = (slash ? slash+1 : m_argv[0]);
	
	for (int i = 1; i < m_argc; ++i) 
	{
		switch (*(end = m_argv[i])) 
		{
			case '-':
				if (!*(++end)) {
					m_error = "Invalid parameter -";
					return false;
				}
				
				switch (*end) 
				{
					case '-':
						char *v;
						
						if (!*(++end)) {
							m_error = "Invalid parameter --";
							return false;
						} else if (*end == '=') {
							m_error = "Invalid parameter --=";
							return false;
						}
						v = strchr(end+1, '=');
						if (v && *(v+1))
							m_params.insert(make_pair(string(end, v-end),
									v+1));
						else if (v)
							m_params.insert(make_pair(string(end, v-end),
									string()));
						else
							m_params.insert(make_pair(end, string()));
						break;
					default:
						for (char p = *(end++); p; p = *(end++)) 
						{	
							if ((i + 1 < m_argc) && *(m_argv[i + 1]) != '-')
								m_params.insert(make_pair(string(1, p),
										m_argv[++i]));
							else
								m_params.insert(make_pair(string(1, p),
										string()));
						}
						break;
				}	
				break;
			
			default:
				m_error = "Invalid parameter ";
				m_error += m_argv[i];
				return false;
		}
	}
	
	char const *const *var = m_envp;
	
	while (*var)
	{
		char *v = strchr(*var, '=');
		if (!v)
			m_vars.insert(make_pair(*var, string()));
		else if (!*(v+1))
			m_vars.insert(make_pair(string(*var, v - *var), string()));
		else
			m_vars.insert(make_pair(string(*var, v - *var), v+1));
		var++;
	}
	
	return eval();
}

/*************************************************************************/

} /* namespace cmdline */

/*************************************************************************/
