/* Quick setup.
 *
 * PegSoft configuration parser library (c) 2004 PegSoft
 * Contact us at pegsoft@pegsoft.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software (the COPYING file); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 */

#include <system/headers.h>

#include "conf/quick.h"

using namespace std;

/*************************************************************************/

namespace conf
{

/*************************************************************************/

ConfQ &ConfQ::operator=(ConfQ const &right)
{
	if (&right != this)
	{
		m_conf = right.m_conf;
		m_setup = right.m_setup;
	}
	
	return *this;
}


/*************************************************************************/

bool ConfQ::parse(ConfGlobalQ &cbq, bool reload, std::istream &is,
		std::string const &context)
{
	if (!m_setup)
		if (!setup())
			return false;
	
	if (!m_conf.begin(cbq.global(), reload))
		return false;
	
	if (!m_conf.parse(is, context))
		return false;
	
	if (!m_conf.end())
		return false;
	
	return true;
}

/*************************************************************************/

bool ConfQ::parse_file(ConfGlobalQ &cbq, bool reload,
		std::string const &filename)
{
	if (!m_setup)
		if (!setup())
			return false;
	
	if (!m_conf.begin(cbq.global(), reload))
		return false;
	
	if (!m_conf.parse_file(filename))
		return false;
	
	if (!m_conf.end())
		return false;
	
	return true;
}

/*************************************************************************/

bool ConfQ::parse_string(ConfGlobalQ &cbq, bool reload,
		std::string const &str, std::string const &context)
{
	if (!m_setup)
		if (!setup())
			return false;
	
	if (!m_conf.begin(cbq.global(), reload))
		return false;
	
	if (!m_conf.parse_string(str, context))
		return false;
	
	if (!m_conf.end())
		return false;
	
	return true;
}

/*************************************************************************/
/*************************************************************************/

DirectivesQ &DirectivesQ::operator=(DirectivesQ const &right)
{
	if (&right != this)
		m_parser = right.m_parser;
	
	return *this;
}

/*************************************************************************/

DirectivesQ::DConfQ &DirectivesQ::DConfQ::operator=
		(DirectivesQ::DConfQ const &right)
{
	if (&right != this)
	{
		ConfQ::operator=(right);
		
		m_directives = right.m_directives;
	}
	
	return *this;
}

/*************************************************************************/

bool DirectivesQ::DConfQ::setup()
{
	if (!m_directives->add(conf()))
		return false;
	
	return ConfQ::setup();
}

/*************************************************************************/

} /* namespace conf */

/*************************************************************************/
