/* Database.
 *
 * PegSoft databases library (c) 2005 PegSoft
 * Contact us at pegsoft@pegsoft.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file COPYING); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <system/headers.h>

#include <modules/core.h>

#include "databases/database.h"
#include "databases/field.h"

using namespace std;

/*************************************************************************/
	
namespace databases
{

/*************************************************************************/

Database::container_type Database::g_list;

/*************************************************************************/

Database::~Database()
{
	g_list.remove(this);
	
	fields_type::iterator it, end;
	
	for (it = m_fields.begin(), end = m_fields.end(); it != end; ++it)
		delete (*it);
}

/*************************************************************************/

Collection *Database::coll(Collection *coll)
{
	if (m_status & dsRegistered)
		return m_coll;
	
	return (m_coll = coll); 
}

/*************************************************************************/

bool Database::delete_(Record const *record)
{
	if (!(m_perms & dpExport) || !writable())
		return false;
	
	return true;
}

/*************************************************************************/

bool Database::delete_owned(Field const *field, Record::identifier_type id)
{
	if (!(m_perms & dpExport) || !writable())
		return false;
	
	return true;
}

/*************************************************************************/

bool Database::field_add(Field *field)
{
	if (m_status & dsRegistered)
		return false;
	
	if (!field || !field->setup())
		return false;
	
	m_fields.push_back(field);
	
	return true;
}

/*************************************************************************/

Field *Database::field_create(char const *name, uint16_t minver, uint16_t type,
		size_t size, uint16_t attrs, char const *def, uint16_t tag)
{
	Field *field = new Field;
	
	field->name(name);
	field->version(vtMinimum, minver);
	field->type(type);
	field->size(size);
	field->attrs_add(attrs);
	field->default_value(def);
	if (tag != 0)
		field->tag(tag);
	
	return field;
}

/*************************************************************************/

Field *Database::field_find(char const *name, int restrict)
{
	fields_type &fields = ((restrict == vtImport) ? m_impfields :
			((restrict == vtExport) ? m_expfields : m_fields));
	
	fields_type::iterator it, end;
	
	for (it = fields.begin(), end = fields.end(); it != end; ++it)
	{
		if (!strcmp((*it)->name(), name))
			return (*it); 
	}
	
	return 0;
}

/*************************************************************************/

Database *Database::find(std::string const &name)
{
	const_iterator it, end;
	
	for (it = begin(), end = Database::end(); it != end; ++it)
		if ((*it)->name() == name)
			return (*it);
	
	return 0;
}

/*************************************************************************/

void Database::generate_expfields()
{
	fields_type::iterator it, end;
	
	for (it = m_fields.begin(), end = m_fields.end(); it != end; ++it)
	{
		if ((version(vtExport) >= (*it)->version(vtMinimum)) &&
				(!(*it)->version(vtMaximum) || 
				(version(vtExport) <= (*it)->version(vtMaximum))))
		{
			m_expfields.push_back(*it);
		}
	}
}

/*************************************************************************/

void Database::generate_impfields()
{
	if (!(m_perms & dpInitial) || (m_status & dsInitial))
	{
		m_impfields = m_expfields;
		return;
	}
	
	fields_type::iterator it, end;
	
	for (it = m_fields.begin(), end = m_fields.end(); it != end; ++it)
	{
		if ((version(vtImport) >= (*it)->version(vtMinimum)) &&
				(!(*it)->version(vtMaximum) || 
				(version(vtImport) <= (*it)->version(vtMaximum))))
		{
			m_impfields.push_back(*it);
		}
	}
}

/*************************************************************************/

bool Database::insert(Record const *record)
{
	if (!(m_perms & dpExport) || !writable())
		return false;
	
	return true;
}

/*************************************************************************/

std::string const &Database::name(std::string const &name)
{
	if (m_status & dsRegistered)
		return m_name;
	
	return (m_name = name);
}

/*************************************************************************/

uint16_t Database::perms_add(uint16_t perms)
{
	if (m_status & dsRegistered)
		return m_perms;
	
	return (m_perms |= perms);
}

/*************************************************************************/

uint16_t Database::perms_remove(uint16_t perms)
{
	if (m_status & dsRegistered)
		return m_perms;
	
	return (m_perms &= ~perms);
}

/*************************************************************************/

bool Database::reg()
{
	g_list.push_back(this);	
	return true;
}

/*************************************************************************/

bool Database::start()
{	
	if (!(m_status & dsRegistered))
	{
		if (!m_coll || m_name.empty() || !m_perms ||
				(!m_expver && (m_perms & dpExport)) ||
				(!m_minver && (m_perms & dpInitial)))
			return false;
		
		generate_expfields();
		/* When there is no initial import needed, generate
		 * the imported fields immediately. */
		if (!(m_perms & dpInitial))
			generate_impfields();
		
		if (!reg())
			return false;
		
		m_status |= dsRegistered;
	}
	
	if (!(m_status & dsStarted))
	{	
		if (!start_real())
			return false;
		
		m_status |= dsStarted;
	}
	
	return true;
}

/*************************************************************************/

bool Database::truncate()
{
	if (!(m_perms & dpExport) || !writable())
		return false;
	
	return true;
}

/*************************************************************************/

bool Database::update_i(Record const *record, Field const *field,
		int32_t value)
{
	if (!(m_perms & dpExport) || !writable())
		return false;
	
	return true;
}

/*************************************************************************/

bool Database::update_multi(Record const *record, unsigned int valcount,
		va_list args)
{
	if (!(m_perms & dpExport) || !writable())
		return false;
	
	return true;
}

/*************************************************************************/

bool Database::update_owned(Field const *field, Record::identifier_type id,
		Record::identifier_type new_id)
{
	if (!(m_perms & dpExport) || !writable())
		return false;
	
	return true;
}

/*************************************************************************/

bool Database::update_ri(Record const *record, Field const *field,
		Record::identifier_type value)
{
	if (!(m_perms & dpExport) || !writable())
		return false;
	
	return true;
}

/*************************************************************************/

bool Database::update_s(Record const *record, Field const *field,
		char const *value)
{
	if (!(m_perms & dpExport) || !writable())
		return false;
	
	return true;
}

/*************************************************************************/

bool Database::update_ui(Record const *record, Field const *field,
		uint32_t value)
{
	if (!(m_perms & dpExport) || !writable())
		return false;
	
	return true;
}

/*************************************************************************/

bool Database::update_ut(Record const *record, Field const *field,
		time_t value)
{
	if (!(m_perms & dpExport) || !writable())
		return false;
	
	return true;
}

/*************************************************************************/

uint16_t Database::version(int type) const
{
	switch (type)
	{
		case vtImport:
			if ((!(m_perms & dpInitial) || (m_status & dsInitial)) &&
					(m_perms & dpExport))
				return m_expver;
			else
				return m_impver;
		case vtExport:
			return m_expver;
		case vtMinimum:
			return m_minver;
		case vtMaximum:
			return m_maxver;
		default:
			return 0;
	}
}

/*************************************************************************/

uint16_t Database::version(int type, uint16_t ver)
{
	if ((m_status & dsRegistered) && type != vtImport)
		return version(type);
	
	switch (type)
	{
		case vtImport:
			return (m_impver = ver);
		case vtExport:
			return (m_expver = ver);
		case vtMinimum:
			return (m_minver = ver);
		case vtMaximum:
			return (m_maxver = ver);
		default:
			return (ver ? 0 : 1);
	};
}

/*************************************************************************/

bool Database::writable() const
{
	return (!(m_core->flags() & modules::Core::pfReadOnly));
}

/*************************************************************************/

} /* namespace databases */

/*************************************************************************/
