/* Log interface.
 *
 * PegSoft log subsystem library (c) 2005 PegSoft
 * Contact us at pegsoft@pegsoft.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software (the COPYING file); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 */

/** 
 * \file log/log.h
 * \brief Log interface header file.
 */

#ifndef LOG_LOG_H
#define LOG_LOG_H

/*************************************************************************/

namespace xlog
{

/*************************************************************************/

class LogLevel;

/*************************************************************************/

/// The log interface.

/** The Log class is the interface used by the Logger class to pass log
  * messages to the registered log modules.
  *
  * This class must be derived by log modules and an object of this
  * class must be passed as argument to the Logger::(un)register_module
  * functions. */

class Log
{
	public:
		/// Destructor.
		virtual ~Log() { }

		/// Logs a message.
		/** Logs a message.
		  *
		  * \param[in] level The log level of the message.
		  * \param[in] message The message to log. */
		virtual void log(LogLevel const &level, char const *message) = 0;
};

/*************************************************************************/

} /* namespace xlog */

/*************************************************************************/

#endif /* LOG_LOG_H */
