/* Mail agent.
 *
 * PegSoft mail library (c) 2006 PegSoft
 * Contact us at pegsoft@pegsoft.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file COPYING); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef MAIL_H
#define MAIL_H

/*************************************************************************/

#include <deque>
#include <string>

#include <misc/ratio.h>

/*************************************************************************/

namespace mail 
{

/*************************************************************************/

class QuotaInfo
{
	public:
		/* Typedefs */
		
		typedef std::deque<time_t> times_list;
		
		/* Functions */
		
		/// Constructor.
		QuotaInfo(std::string const &name) : m_name(name), m_times() { }
		
		/// Checks whether we're exceeding quota.
		bool exceeds(misc::Ratio const &ratio);
		/// Adds a mark to the quota.
		void mark();
		
		/// Returns the name.
		std::string const &name() const { return m_name; }
		/// Returns the times list.
		times_list const &times() const { return m_times; }
	private:
		std::string m_name;
		times_list m_times;
		
		QuotaInfo(QuotaInfo const &);
		QuotaInfo &operator=(QuotaInfo const &);
		
		/// Removes too old entries in the list.
		void purge(misc::Ratio const &ratio);
};

/*************************************************************************/

} /* namespace mail */

/*************************************************************************/

#endif /* MAIL_H */
