/* recursive_mkdir().
 *
 * PegSoft miscellaneous library (c) 2004 PegSoft
 * Contact us at pegsoft@pegsoft.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software (the COPYING file); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 */

#include <system/headers.h>

#if HAVE_SYS_STAT_H
# include <sys/stat.h>
#endif

#include <string>

#include "misc/misc.h"

using namespace std;

/*************************************************************************/

namespace misc
{

/*************************************************************************/

static bool recurse(int &rcount, char const *pathname, mode_t mode)
{
	/* Protection against too much recursions. */
	if (rcount++ == 128)
		return false;
	
	struct stat st;
	
	if (stat(pathname, &st) == 0)
	{
		/* The path exists. Check if it is a directory, and if
		 * it's not, return with an error. */
		if (!S_ISDIR(st.st_mode))
		{
#ifdef ENOTDIR
			errno = ENOTDIR;
#endif
			return false;
		}
	} else {
		/* Check why the stat call failed, and stop if it is a
		 * fatal error. */
		 if (errno != ENOENT)
		 	return false;
		
		/* The directory does not exist!
		 * Now try to call recurse() on the parent directory first. */
		
		char const *end = &(pathname[strlen(pathname) - 1]);
		
		/* Remove trailing slashes */
	
		while (*end == '/')
		{
			/* This shouldn't happen unless something goes
			 * very wrong (because the name should either exist
			 * or we wouldn't have got a ENOENT before). */
			if (end == pathname)
				return false;
			--end;
		}
		
		/* Find the first slash from the end */
		
		while (*end != '/')
		{
			/* If that happens, then the first part of the path is
			 * relative, so create the directory. */
			if (end == pathname)
			{
				if (mkdir(pathname, mode) == -1)
					return false;
				return true;
			}
			
			--end;
		}
		
		if (!recurse(rcount, std::string(pathname, end + 1).c_str(), mode))
			return false;
		
		/* Okay, now we can create the directory. */
		if (mkdir(pathname, mode) == -1)
			return false;
	}
	
	return true;
}

/*************************************************************************/

bool recursive_mkdir(char const *pathname, mode_t mode)
{
	size_t len;
	
	/* Make sure we aren't going to process something crazy. */
	if (!pathname)
	{
		errno = EFAULT;
		return false;
	}
	
	if (!*pathname)
	{
		errno = EINVAL;
		return false;
	}
	
	if ((len = strlen(pathname)) > PATH_MAX)
	{
#ifdef ENAMETOOLONG
		errno = ENAMETOOLONG;
#endif
		return false;
	}
	
	int rcount = 0;
	return recurse(rcount, pathname, mode);
}

/*************************************************************************/

} /* namespace misc */

/*************************************************************************/
