/* Resolving stuff.
 *
 * PegSoft miscellaneous library (c) 2004 PegSoft
 * Contact us at pegsoft@pegsoft.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software (the COPYING file); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 */

#include <system/headers.h>

#if HAVE_SYS_SOCKET_H
# include <sys/socket.h>
#endif
#if HAVE_ARPA_INET_H
# include <arpa/inet.h>
#endif
#if HAVE_NETDB_H
# include <netdb.h>
#endif

#ifdef USE_THREADS
# if !HAVE_GETADDRINFO && HAVE_GETHOSTBYNAME && !HAVE_GETHOSTBYNAME_R3 && !HAVE_GETHOSTBYNAME_R5 && !HAVE_GETHOSTBYNAME_R6
#  include <pthread.h>
# endif
#endif

#include "misc/misc.h"
#include "misc/string.h"

using namespace std;

/*************************************************************************/

namespace
{

#ifdef USE_THREADS
# if !HAVE_GETADDRINFO && HAVE_GETHOSTBYNAME && !HAVE_GETHOSTBYNAME_R3 && !HAVE_GETHOSTBYNAME_R5 && !HAVE_GETHOSTBYNAME_R6
pthread_mutex_t resolve_mutex = PTHREAD_MUTEX_INITIALIZER;
# endif
#endif

}

/*************************************************************************/

namespace misc
{

/*************************************************************************/

bool resolve_error(char *buf, size_t len, int err)
{
#if HAVE_GETADDRINFO || HAVE_GETHOSTBYNAME
	if (err == 0)
		return false;
# if HAVE_GETADDRINFO
#  ifdef EAI_SYSTEM
     if (err == EAI_SYSTEM)
     	snprintf(buf, len, "System error: %s", strerror(errno));
#  endif
	strncpyzt(buf, gai_strerror(err), len);
# elif HAVE_GETHOSTBYNAME
    strncpyzt(buf, hstrerror(err), len);
# endif
	return true;
#else
	return false;
#endif
}

/*************************************************************************/

bool resolve_host(char const *host, struct in_addr *in, int *err)
{
#if HAVE_GETADDRINFO
	struct addrinfo hints, *ai;
	
	memset(&hints, 0, sizeof(hints));
	hints.ai_family = PF_INET;
	
	int my_err = getaddrinfo(host, NULL, &hints, &ai);
	
	if (my_err) {
		if (err)
			*err = my_err;
		return false;
	}
	
	if (err)
		*err = 0;
	
	struct sockaddr_in *sin;	
	sin = reinterpret_cast<struct sockaddr_in *> (ai->ai_addr);
	memcpy(in, &sin->sin_addr, sizeof(sin->sin_addr));
	
	freeaddrinfo(ai);
	return true;
#elif HAVE_GETHOSTBYNAME
	struct hostent *hentp = NULL;

# ifdef USE_THREADS
#  if HAVE_GETHOSTBYNAME_R3
	struct hostent hent;
	struct hostent_data data;
	
	hentp = gethostbyname_r(host, &hent, &data);
	if (err)
		*err = (!hentp ? h_errno : 0);
#  elif HAVE_GETHOSTBYNAME_R5
	struct hostent hent;
	char hbuf[8192];
	int my_herrno;
	
	hentp = gethostbyname_r(host, &hent, hbuf, sizeof(hbuf), &my_herrno);
	if (err)
		*err = (!hentp ? my_herrno : 0);
#  elif HAVE_GETHOSTBYNAME_R6
	int my_herrno;
	struct hostent hent;
	char hbuf[16384];
	
	gethostbyname_r(host, &hent, hbuf, sizeof(hbuf), &hentp, &my_herrno); 
	if (err)
		*err = (!hentp ? my_herrno : 0);
#  else
	if (pthread_mutex_lock(&resolve_mutex))
		return false;
	hentp = gethostbyname(host);
	if (err)
		*err = (!hentp ? h_errno : 0);
#  endif
# else
	hentp = gethostbyname(host);
	if (err)
		*err = (!hentp ? h_errno : 0);
# endif
	if (hentp)
		memcpy(&in->s_addr, hentp->h_addr_list[0], hentp->h_length);

# ifdef USE_THREADS
#  if !HAVE_GETHOSTBYNAME_R3 && !HAVE_GETHOSTBYNAME_R5 && !HAVE_GETHOSTBYNAME_R6
	pthread_mutex_unlock(&resolve_mutex);
#  endif
# endif
	
	return (hentp ? true : false);
#else
	if (err)
		*err = 0;
	return (inet_aton(host, in) ? true : false);
#endif
}

/*************************************************************************/

} /* namespace misc */

/*************************************************************************/
