/* Core module.
 *
 * PegSoft modules subsystem library (c) 2005 PegSoft
 * Contact us at pegsoft@pegsoft.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software (the COPYING file); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 */

#include <system/headers.h>

#include <log/logger.h>

#include "modules/core.h"

using namespace std;
using namespace xlog;

/*************************************************************************/

namespace modules
{

/*************************************************************************/

Core::Core() : Module("core", "The core"), m_action(0),
		m_debuglevel(0), m_flags(0), m_started(false), m_starttime(time(0))
{
	/* Initialize exit message */
	memset(exitmsg, 0, sizeof(exitmsg));
	
	/* Sets the state */
	m_state |= msLoaded;
}

/*************************************************************************/

void Core::fatalf(char const *msg, ...)
{
	if (msg)
	{
		va_list args;
		va_start(args, msg);
		vsnprintf(exitmsg, exitmsg_size, msg, args);
		va_end(args);
	}
	
	fatal();
}

/*************************************************************************/

void Core::log(xlog::LogLevel const &level, char const *format, ...)
{
	va_list args;
	va_start(args, format);
	logger().vlog(level, format, args);
	va_end(args);
}

/*************************************************************************/

void Core::log_perror(xlog::LogLevel const &level, char const *format, ...)
{
	va_list args;
	va_start(args, format);
	
	if (errno != 0)
	{
		int my_errno = errno;
		char msg[Logger::msg_length], *end = msg;
	
		end += vsnprintf(msg, sizeof(msg) - 256, format, args);
	
		/* Add the system error message. */
		snprintf(end, (sizeof(msg) - (end - msg)), ": %s", strerror(my_errno));
	
		logger().log(level, "%s", msg);
	} else {
		logger().vlog(level, format, args);
	}
	
	va_end(args);
}

/*************************************************************************/

} /* namespace modules */

/*************************************************************************/
