/* Vaste programme.
 *
 * PegSoft scheduler library (c) 2005 PegSoft
 * Contact us at pegsoft@pegsoft.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software (the COPYING file); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 */

#include <system/headers.h>

#include "scheduler/scheduler.h"

using namespace std;

/*************************************************************************/

namespace scheduler
{

/*************************************************************************/

Scheduler &Scheduler::operator=(Scheduler const &right)
{
	if (&right != this)
	{
		m_tasks = right.m_tasks;
		m_curtask = 0;
		m_tasktype = right.m_tasktype;
	}
	
	return *this;
}

/*************************************************************************/

bool Scheduler::add(Task &task)
{
	list_type::size_type size = m_tasks.size();
	
	m_tasks.push_back(&task);
	
	if (m_tasks.size() > size)
		return true;
	else
		return false;
}

/*************************************************************************/

bool Scheduler::remove(Task &task)
{
	list_type::size_type size = m_tasks.size();
	
	m_tasks.remove(&task);
	
	if (m_tasks.size() < size)
		return true;
	else
		return false;
}

/*************************************************************************/

Scheduler::list_type::size_type Scheduler::remove_byname(int type,
		string const &name)
{
	list_type::size_type count = 0;
	list_type::iterator it, end, cur;
	
	for (it = m_tasks.begin(), end = m_tasks.end(); it != end;)
	{
		cur = it++;
		
		if ((*cur)->type() == type && (*cur)->name() == name)
		{
			m_tasks.erase(cur);
			delete (*cur);
			++count;
		}
	}
	
	return count;
}

/*************************************************************************/

Scheduler::list_type::size_type Scheduler::remove_bytype(int type)
{
	list_type::size_type count = 0;
	list_type::iterator it, end, cur;
	
	for (it = m_tasks.begin(), end = m_tasks.end(); it != end;)
	{
		cur = it++;
		
		if ((*cur)->type() == type)
		{
			m_tasks.erase(cur);
			delete (*cur);
			++count;
		}
	}
	
	return count;
}

/*************************************************************************/

bool Scheduler::run()
{
	bool retval = true;
	list_type::iterator it, end, next;
	
	for (it = m_tasks.begin(), end = m_tasks.end(); it != end; it = next)
	{	
		if ((*it)->poll()) 
		{
			if (!run_task(it, &next))
				retval = false;
		} else {
			next = ++it;
		}
	}
	
	return retval;
}

/*************************************************************************/

Scheduler::list_type::size_type Scheduler::run_byname(int type,
		string const &name)
{
	list_type::size_type count = 0;
	list_type::iterator it, end, next;
	
	for (it = m_tasks.begin(), end = m_tasks.end(); it != end; it = next)
	{
		if (((*it)->type() == type) && ((*it)->name() == name))
		{
			run_task(it, &next);
			++count;
		} else {
			next = ++it;
		}
	}
	
	return count;
}

/*************************************************************************/

Scheduler::list_type::size_type Scheduler::run_bytype(int type)
{
	list_type::size_type count = 0;
	list_type::iterator it, end, next;
	
	for (it = m_tasks.begin(), end = m_tasks.end(); it != end; it = next)
	{
		if (((*it)->type() == type))
		{
			run_task(it, &next);
			++count;
		} else {
			next = ++it;
		}
	}
	
	return count;
}

/*************************************************************************/

bool Scheduler::run_task(list_type::iterator &task, list_type::iterator *next)
{
	Task *prev_task = m_curtask;
	m_curtask = *task;
	int res = (*task)->run();
	m_curtask = prev_task;
	
	*next = ++task;
	--task;
	
	if (res & Task::rRemove)
	{
		m_tasks.erase(task);
		if (res == Task::rFree)
			delete (*task);
	}
	
	if (res & Task::rSlept)
		return false;
	
	return true;
}

/*************************************************************************/

Task &Task::operator=(Task const &right)
{
	if (&right != this)
	{
		m_type = right.m_type;
	}
	
	return *this;
}

/*************************************************************************/

TaskStep::TaskStep(int type, time_t step, time_t shift) : Task(type),
		m_step(step), m_shift(shift), m_next(0)
{
	calc_next();
}

/*************************************************************************/

TaskStep &TaskStep::operator=(TaskStep const &right)
{
	if (&right != this)
	{
		Task::operator=(right);
		
		m_step = right.m_step;
		m_shift = right.m_shift;
		m_next = right.m_next;
	}
	
	return *this;
}

/*************************************************************************/

void TaskStep::calc_next()
{	
	time_t now = time(0), offset = timezone_offset(now);
	if (offset < m_step)
		now += offset;
	
	m_next = ((now / m_step) * m_step) + (m_step + m_shift);
	
	offset = timezone_offset(m_next);
	if (offset < m_step)
		m_next -= timezone_offset(m_next);
}

/*************************************************************************/

time_t TaskStep::timezone_offset(time_t now)
{
	struct tm gtm = *gmtime(&now);
	struct tm ltm = *localtime(&now);
	
	time_t offset = 0;
	
	if (ltm.tm_year > gtm.tm_year)
		ltm.tm_yday += (gtm.tm_yday + 1);
	else if (gtm.tm_year > ltm.tm_year)
		gtm.tm_yday += (ltm.tm_yday + 1);
	
	offset += ((ltm.tm_yday - gtm.tm_yday) * 86400);
	offset += ((ltm.tm_hour - gtm.tm_hour) * 3600);
	offset += ((ltm.tm_min  - gtm.tm_min)  * 60);
	offset +=  (ltm.tm_sec  - gtm.tm_sec);
	
	return offset;
}

/*************************************************************************/

bool TaskStep::poll()
{
	time_t now = time(0);
	
	if (now >= m_next) {
		calc_next();
		return true;
	} else {
		return false;
	}
}

/*************************************************************************/

time_t TaskStep::shift(time_t shift)
{
	if (m_shift != shift)
	{
		m_shift = shift;
		calc_next();
	}
	
	return m_shift;
}

/*************************************************************************/

time_t TaskStep::step(time_t step)
{
	if (m_step != step)
	{
		m_step = step;
		calc_next();
	}
	
	return m_step;
}

/*************************************************************************/

TaskTimeout &TaskTimeout::operator=(TaskTimeout const &right)
{
	if (&right != this)
	{
		Task::operator=(right);
		
		m_timeout = right.m_timeout;
		m_lastrun = right.m_lastrun;
	}
	
	return *this;
}

/*************************************************************************/

bool TaskTimeout::poll()
{
	time_t now = time(NULL);
	
	if ((now - m_timeout) >= m_lastrun) {
		m_lastrun = now;
		return true;
	} else {
		return false;
	}
}

/*************************************************************************/

} /* namespace scheduler */

/*************************************************************************/
