/* A simple strsignal() implementation.
 *
 * PegSoft compatibility library (c) 2004 PegSoft
 * Contact us at pegsoft@pegsoft.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software (the COPYING file); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 */

#include "system/headers.h"
#include "system/compat.h"

#if HAVE_SIGNAL_H
# include <signal.h>
#endif

/*************************************************************************/

char *compat_strsignal(int sig)
{
#ifdef SYS_SIGLIST_DECLARED
	return sys_siglist[sig];
#else
	static char sigbuf[64];
	
	switch (sig) {
		case SIGHUP:
			strcpy(sigbuf, "Hangup");
			break;
		
		case SIGINT:
			strcpy(sigbuf, "Interrupt");
			break;
		
		case SIGQUIT:
			strcpy(sigbuf, "Quit");
			break;
		
		case SIGILL:
			strcpy(sigbuf, "Illegal instruction");
			break;
		
		case SIGABRT:
			strcpy(sigbuf, "Aborted");
			break;
		
		case SIGFPE:
			strcpy(sigbuf, "Floating point exception");
			break;
		
		case SIGKILL:
			strcpy(sigbuf, "Killed");
			break;
		
		case SIGSEGV:
			strcpy(sigbuf, "Segmentation fault");
			break;
		
		case SIGPIPE:
			strcpy(sigbuf, "Broken pipe");
			break;
		
		case SIGALRM:
			strcpy(sigbuf, "Alarm clock");
			break;
		
		case SIGTERM:
			strcpy(sigbuf, "Terminated");
			break;
		
		case SIGUSR1:
			strcpy(sigbuf, "User defined signal 1");
			break;
		
		case SIGUSR2:
			strcpy(sigbuf, "User defined signal 2");
			break;
		
		case SIGCHLD:
			strcpy(sigbuf, "Child exited");
			break;
		
		case SIGCONT:
			strcpy(sigbuf, "Continued");
			break;
		
		case SIGSTOP:
			strcpy(sigbuf, "Stopped (signal)");
			break;
		
		case SIGTSTP:
			strcpy(sigbuf, "Stopped");
			break;
		
		case SIGTTIN:
			strcpy(sigbuf, "Stopped (tty input)");
			break;
		
		case SIGTTOU:
			strcpy(sigbuf, "Stopped (tty output)");
			break;
		
#ifdef SIGBUS
		case SIGBUS:
			strcpy(sigbuf, "Bus error");
			break;
#endif

#ifdef SIGPOLL
		case SIGPOLL:
			strcpy(sigbuf, "Pollable event");
			break;
#endif

#ifdef SIGPROF
		case SIGPROF:
			strcpy(sigbuf, "Profiling timer expired");
			break;
#endif

#ifdef SIGSYS
		case SIGSYS:
			strcpy(sigbuf, "Bad system call");
			break;
#endif

#ifdef SIGTRAP
		case SIGTRAP:
			strcpy(sigbuf, "Trace/breakpoint trap");
			break;
#endif

#ifdef SIGURG
		case SIGURG:
			strcpy(sigbuf, "Urgent I/O condition");
			break;
#endif

#ifdef SIGVTALRM
		case SIGVTALRM:
			strcpy(sigbuf, "Virtual timer expired");
			break;
#endif

#ifdef SIGXCPU
		case SIGXCPU:
			strcpy(sigbuf, "CPU time limit exceeded");
			break;
#endif

#ifdef SIGXFSZ
		case SIGXFSZ:
			strcpy(sigbuf, "File size limit exceeded");
			break;
#endif

#if defined(SIGIOT) && SIGIOT != SIGABRT
		case SIGIOT:
			strcpy(sigbuf, "IOT trap");
			break;
#endif

#ifdef SIGEMT
		case SIGEMT:
			strcpy(sigbuf, "???");
			break;
#endif

#ifdef SIGSTKFLT
		case SIGSTKFLT:
			strcpy(sigbuf, "Stack fault");
			break;
#endif

#if defined(SIGIO) && (!defined(SIGPOLL) || SIGIO != SIGPOLL)
		case SIGIO:
			strcpy(sigbuf, "I/O possible");
			break;
#endif

#if defined(SIGCLD) && SIGCHLD != SIGCLD
		case SIGCLD:
			strcpy(sigbuf, "Child exited");
			break;
#endif

#ifdef SIGPWR
		case SIGPWR:
			strcpy(sigbuf, "Power failure");
			break;
#endif

#ifdef SIGINFO
		case SIGINFO:
			strcpy(sigbuf, "Power failure");
			break;
#endif

#ifdef SIGLOST
		case SIGLOST:
			strcpy(sigbuf, "File lock lost");
			break;
#endif

#ifdef SIGWINCH
		case SIGWINCH:
			strcpy(sigbuf, "Window changed");
			break;
#endif
		
		default:
			sprintf(sigbuf, "Unknown signal %d", sig);
			break;
	}
	
	return sigbuf;
#endif /* SYS_SIGLIST_DECLARED */
}

/*************************************************************************/
