/* Various routines to perform simple actions.
 *
 * Epona (c) 2000-2002, 2004 PegSoft
 * Contact us at epona@pegsoft.net
 *
 * This program is free but copyrighted software; see the file COPYING for
 * details.
 *
 * Based on the original code of Services by Andy Church.
 *
 */

#include <system/headers.h>

#include <core/directives.h>
#include <core/epona.h>

#include "epona.h"

#include "services.h"

using namespace std;

/*************************************************************************/

/* Note a bad password attempt for the given user.  If they've used up
 * their limit, toss them off.  
 */

void bad_password(User *u)
{
	if (!(epona->cd().bad_passwords))
		return;

	time_t now = time(NULL);

	if ((epona->cd().bad_passwords.timeout > 0) && (u->invalid_pw_time > 0) &&
			(u->invalid_pw_time < (now - epona->cd().bad_passwords.timeout)))
		u->invalid_pw_count = 0;
    
	u->invalid_pw_count++;
	u->invalid_pw_time = now;
    
	if (u->invalid_pw_count >= epona->cd().bad_passwords.limit)
#ifdef IRC_BAHAMUT
		send_cmd(NULL, "SVSKILL %s :%s", u->nick, "Too many invalid passwords");
#else
		kill_user(NULL, u->nick, "Too many invalid passwords");
#endif
}

/*************************************************************************/

void change_user_mode(char const *source, User *u,
		char const *modes, char const *arg) 
{
	int ac = 1;
	char const *av[2];
	
	av[0] = modes;
	if (arg) {
		av[1] = arg;
		ac++;
	}
	
	s_svsmode_user(source, u->nick, u->timestamp, modes, arg);
	set_umode(u, ac, av);
}

/*************************************************************************/

/* Remove a user from the IRC network.  `source' is the nick which should
 * generate the kill, or NULL for a server-generated kill.
 */

void kill_user(const char *source, const char *user, const char *reason)
{
    char const *av[2];
    char buf[BUFSIZE];

    if (!user || !*user)
	return;
    if (!source || !*source)
	source = epona->cd().services.sname;
    if (!reason)
	reason = "";
    snprintf(buf, sizeof(buf), "%s (%s)", source, reason);
    av[0] = user;
    av[1] = buf;
    send_cmd(source, "KILL %s :%s", user, av[1]);
    do_kill(source, 2, av);
}

/*************************************************************************/

void set_topic(char const *source, Channel *c, char const *topic,
		char const *setter, time_t when)
{	
	if (!setter)
		setter = source;
	
#if defined(IRC_DREAMFORGE) && !defined(IRC_ULTIMATE) && !defined(IRC_UNREAL)
	if (c->topic_time && when >= c->topic_time)
		when = c->topic_time - 1; /* Newer timestamps are rejected */
#endif 
#if defined(IRC_UNREAL)
	if (when <= c->topic_time)
		when = c->topic_time + 1; /* Older timestamps are rejected */
#endif
	
	if ((!topic && !c->topic) || (topic && c->topic && !strcmp(topic, c->topic)))
		if (!strcmp(setter, c->topic_setter) && c->topic_time == when)
			return;
	
	if (c->topic)
		free(c->topic);
	
	if (topic && *topic)
		c->topic = sstrdup(topic);
	else
		c->topic = 0;
	
	strscpy(c->topic_setter, setter, NICKMAX);
	
	c->topic_time = when;
	
	send_cmd(source, "TOPIC %s %s %ld :%s", c->name, c->topic_setter,
    		(long) c->topic_time, c->topic ? c->topic : "");
}

/*************************************************************************/
