/* MemoServ functions.
 *
 * Epona (c) 2000-2002, 2004 PegSoft
 * Contact us at epona@pegsoft.net
 *
 * This program is free but copyrighted software; see the file COPYING for
 * details.
 *
 * Based on the original code of Services by Andy Church.
 *
 */

#include <system/headers.h>

#include <core/directives.h>
#include <core/epona.h>
#include <mail/mail.h>

#include "chanserv.h"
#include "epona.h"
#include "memoserv.h"
#include "nickserv.h"
#include "operserv.h"

#include "services.h"
#include "pseudo.h"

using namespace std;
using namespace core;

/*************************************************************************/

static int delmemo(char const *target, MemoInfo *mi, uint32_t num);

static void do_help(User *u);
static void do_send(User *u);
static void do_cancel(User *u);
static void do_list(User *u);
static void do_read(User *u);
static void do_del(User *u);
static void do_set(User *u);
static void do_set_notify(User *u, MemoInfo *mi, char *param);
static void do_set_limit(User *u, MemoInfo *mi, char *param);
static void do_set_forward(User *u, MemoInfo *mi, char *param);
static void do_info(User *u);

/*************************************************************************/

static Command cmds[] = {
    { "HELP",        do_help, 	NULL,  -1,                      -1,-1,-1,-1 },
    { "SEND",        do_send, 	NULL,  MEMO_HELP_SEND,          -1,-1,-1,-1 },
    { "CANCEL",      do_cancel, 	NULL,  MEMO_HELP_CANCEL,        -1,-1,-1,-1 },
    { "LIST",        do_list, 	NULL,  MEMO_HELP_LIST,          -1,-1,-1,-1 },
    { "READ",        do_read, 	NULL,  MEMO_HELP_READ,          -1,-1,-1,-1 },
    { "DEL",         do_del,  	NULL,  MEMO_HELP_DEL,           -1,-1,-1,-1 },
    { "SET",         do_set,  	NULL,  MEMO_HELP_SET,           -1,-1,-1,-1 },
    { "SET NOTIFY",  NULL,    	NULL,  MEMO_HELP_SET_NOTIFY,    -1,-1,-1,-1 },
    { "SET LIMIT",   NULL,    	NULL,  -1,     
    	MEMO_HELP_SET_LIMIT, MEMO_SERVADMIN_HELP_SET_LIMIT,
    	MEMO_SERVADMIN_HELP_SET_LIMIT, MEMO_SERVADMIN_HELP_SET_LIMIT },
    { "SET FORWARD", NULL,    	NULL,  MEMO_HELP_SET_FORWARD,   -1,-1,-1,-1 },
    { "INFO",        do_info, NULL,  -1,
		MEMO_HELP_INFO, MEMO_SERVADMIN_HELP_INFO,
		MEMO_SERVADMIN_HELP_INFO, MEMO_SERVADMIN_HELP_INFO },
    { NULL }
};

/*************************************************************************/
/*************************************************************************/

/* MemoServ initialization. */

void ms_init(void)
{
	Command *cmd;
	
	cmd = lookup_cmd(cmds, "SEND");
	if (cmd)
		cmd->help_param1 = s_ChanServ;
	
	cmd = lookup_cmd(cmds, "SET LIMIT");
	if (cmd)
		cmd->help_param1 = (char *)(long)epona->cd().memoserv.max_memos;
	
	cmd = lookup_cmd(cmds, "SET FORWARD");
	if (cmd)
	{
		cmd->help_param1 = s_NickServ;
		cmd->help_param2 = s_NickServ;
	}
}

/*************************************************************************/

/* memoserv:  Main MemoServ routine.
 *            Note that the User structure passed to the do_* routines will
 *            always be valid (non-NULL) and will always have a valid
 *            NickInfo pointer in the `ni' field.
 */

void memoserv(User *u, char *buf)
{
    char *cmd, *s;

    cmd = strtok(buf, " ");
    if (!cmd) {
		return;
    } else if (stricmp(cmd, "\1PING") == 0) {
		if (!(s = strtok(NULL, "")))
	    	s = "\1";
		notice(s_MemoServ, u->nick, "\1PING %s", s);
    } else {
		if (!u->na && stricmp(cmd, "HELP") != 0)
	    	notice_lang(s_MemoServ, u, NICK_NOT_REGISTERED_HELP, s_NickServ);
		else
	    	run_cmd(s_MemoServ, u, cmds, cmd);
    }
}

/*************************************************************************/

/* check_memos:  See if the given user has any unread memos, and send a
 *               NOTICE to that user if so (and if the appropriate flag is
 *               set).
 */

void check_memos(User *u)
{
	NickGroup *ng;
	int i, newcnt = 0;
	
	if (!(ng = (u->na ? u->na->ng() : NULL)) || !nick_recognized(u) ||
			!(ng->flags() & NickGroup::gfMemoSignon))
		return;
	
	for (i = 0; i < ng->memos.memos().size(); i++) 
	{
		if (ng->memos.memos()[i]->flags() & Memo::mfUnread)
		    newcnt++;
	}
	
	if (newcnt > 0) {
		notice_lang(s_MemoServ, u,
				newcnt==1 ? MEMO_HAVE_NEW_MEMO : MEMO_HAVE_NEW_MEMOS, newcnt);
		if (newcnt == 1 && (ng->memos.memos()[i-1]->flags() & Memo::mfUnread)) {
		    notice_lang(s_MemoServ, u, MEMO_TYPE_READ_LAST, s_MemoServ);
		} else if (newcnt == 1) {
	    	for (i = 0; i < ng->memos.memos().size(); i++) {
				if (ng->memos.memos()[i]->flags() & Memo::mfUnread)
				    break;
	    	}
	    	notice_lang(s_MemoServ, u, MEMO_TYPE_READ_NUM, s_MemoServ,
					ng->memos.memos()[i]->number());
		} else {
		    notice_lang(s_MemoServ, u, MEMO_TYPE_LIST_NEW, s_MemoServ);
		}
	}
	
	if (ng->maxmemos() > 0 && ng->memos.memos().size() >= ng->maxmemos()) 
	{
		if (ng->memos.memos().size() > ng->maxmemos())
		    notice_lang(s_MemoServ, u, MEMO_OVER_LIMIT, ng->maxmemos());
		else
		    notice_lang(s_MemoServ, u, MEMO_AT_LIMIT, ng->maxmemos());
	}
}

/*************************************************************************/
/*********************** MemoServ private routines ***********************/
/*************************************************************************/

static bool addmemo(User *sender, NickGroup *recipient, char const *target,
		char const *text)
{
	MemoInfo *mi = &recipient->memos;
	
	Memo *m = (Memo *)memos.create();
	
	m->owner(recipient);
	
	if (!mi->memos().empty()) 
		m->number(mi->memos().back()->number() + 1);
	else
		m->number(1);
	
	m->sender(sender->na->ng()->name());
	if (target)
		m->target(target);
	m->text(text);
	
	if (!memos.add(m, 0))
	{
		delete m;
		return false;
	}
	
	return true;
}

/*************************************************************************/

/* Delete a memo by number.  Return 1 if the memo was found, else 0. */

static int delmemo(char const *target, MemoInfo *mi, uint32_t num)
{
	MemoInfo::list_type::iterator it, end;
	
	for (it = mi->memos().begin(), end = mi->memos().end(); it != end; ++it)
	{
		Memo *m = (*it);
		
		if (m->number() != num)
			continue;

		if (!memos.remove(m, 0))
			return 0;
		
		return 1;
	}
	
	return 0;
}

/*************************************************************************/
/*********************** MemoServ command routines ***********************/
/*************************************************************************/

/* Return a help message. */

static void do_help(User *u)
{
	char *cmd = strtok(NULL, "");
	
	if (!cmd) {
		notice_help(s_MemoServ, u, MEMO_HELP, s_ChanServ);
	} else {
		help_cmd(s_MemoServ, u, cmds, cmd);
	}
}

/*************************************************************************/

/* Send a memo to a nick/channel. */

static void do_send(User *u)
{
	NickAlias *na;
	ChannelInfo *ci = 0;
	
	char *name = strtok(NULL, " ");
	char *text = strtok(NULL, "");
	time_t now = time(0);
	int is_servadmin = is_services_admin(u);
	
	if (!memos.writable()) {
		notice_lang(s_MemoServ, u, MEMO_SEND_DISABLED);
	} else if (!text) {
		syntax_error(s_MemoServ, u, "SEND", MEMO_SEND_SYNTAX);
	} else if (!nick_recognized(u)) {
		notice_lang(s_MemoServ, u, NICK_IDENTIFY_REQUIRED, s_NickServ);
	} else if (epona->cd().nickserv.auth &&
			epona->cd().memoserv.restrict_auth.send() &&
			!(u->na->ng()->flags() & NickGroup::gfAuth)) {
		notice_lang(s_MemoServ, u, MEMO_SEND_NOT_AUTH);
		if (!u->na->ng()->auth_code())
			notice_lang(s_MemoServ, u,
					MEMO_SEND_NOT_AUTH_HOWTO, s_NickServ);
	} else if ((epona->cd().memoserv.send_delay > 0) &&
			(u->lastmemosend + epona->cd().memoserv.send_delay > now) &&
			!is_servadmin) {
		u->lastmemosend = now;
		notice_lang(s_MemoServ, u, MEMO_SEND_PLEASE_WAIT,
				(uint16_t)epona->cd().memoserv.send_delay);
	} else if (*name == '#' && !epona->cd().memoserv.channel_target) {
		notice_lang(s_MemoServ, u, MEMO_SEND_CHANNEL_TARGET_DISABLED);
	} else if (!(na = aliases.find(name)) && !(ci = channels.find(name))) {
		notice_lang(s_MemoServ, u, ((*name != '#') ? NICK_X_NOT_REGISTERED :
				CHAN_X_NOT_REGISTERED), name);
	} else if (na && epona->cd().nickserv.auth &&
			epona->cd().memoserv.restrict_auth.get() &&
			!(na->ng()->flags() & NickGroup::gfAuth)) {
		notice_lang(s_MemoServ, u, MEMO_SEND_TARGET_NOT_AUTH, na->nick());
	} else if (ci && !check_access(u, ci, CA_MEMO)) {
		notice_lang(s_MemoServ, u, PERMISSION_DENIED);
	} else if (na) 
	{
		if (na->ng()->maxmemos() == 0 && !is_servadmin) {
			notice_lang(s_MemoServ, u, MEMO_X_GETS_NO_MEMOS, name);
		} else if (na->ng()->maxmemos() > 0 &&
				na->ng()->memos.memos().size() >= na->ng()->maxmemos() &&
				!is_servadmin) {
			notice_lang(s_MemoServ, u, MEMO_X_HAS_TOO_MANY_MEMOS, name);
		} else {
			if (addmemo(u, na->ng(), 0, text))
			{
				u->lastmemosend = now;
				notice_lang(s_MemoServ, u, MEMO_SENT, name);
			} else {
				notice_lang(s_MemoServ, u, MEMO_SEND_FAILED);
			}
		}
	} else if (ci)
	{
		if ((ci->owner()->maxmemos() != 0) || is_servadmin)
		{	
			if (epona->cd().nickserv.auth &&
					epona->cd().memoserv.restrict_auth.get() &&
					!(ci->owner()->flags() & NickGroup::gfAuth)) 
			{
				notice_lang(s_MemoServ, u,
						MEMO_SEND_TARGET_NOT_AUTH, ci->owner()->name());
			} 
			else if ((ci->owner()->maxmemos() > 0) &&
					(ci->owner()->memos.memos().size() >=
					 ci->owner()->maxmemos()) && !is_servadmin) 
			{
				notice_lang(s_MemoServ, u,
						MEMO_X_HAS_TOO_MANY_MEMOS, ci->owner()->name());
			} else {
				addmemo(u, ci->owner(), name, text);
			}
		}
		
		Access *acc;
					
		ChannelInfo::accesses_type::const_iterator it, end;
	
		for (it = ci->accesses().begin(),
				end = ci->accesses().end(); it != end; ++it)
		{
			acc = (Access *) (*it);
		
			if (!acc)
				continue;
			
			if (acc->level() >= ci->levels(CA_MEMO))
			{
				NickGroup *ng = acc->ng();
				
				if (ng == ci->owner())
					continue;
				
				if (epona->cd().nickserv.auth &&
						epona->cd().memoserv.restrict_auth.get() &&
						!(ng->flags() & NickGroup::gfAuth)) 
				{
					notice_lang(s_MemoServ, u,
							MEMO_SEND_TARGET_NOT_AUTH, ng->name());
					continue;
				}
				
				if ((ng->maxmemos() == 0) && !is_servadmin)
					continue;
				
				if ((ng->maxmemos() > 0) &&
						(ng->memos.memos().size() >= ng->maxmemos()) &&
						!is_servadmin)
				{
					notice_lang(s_MemoServ, u,
							MEMO_X_HAS_TOO_MANY_MEMOS, ng->name());
					continue;
				}
				
				if (!addmemo(u, ng, name, text))
				{
					notice_lang(s_MemoServ, u, MEMO_SEND_FAILED);
					return;
				}
			}
		}
		
		u->lastmemosend = now;
		notice_lang(s_MemoServ, u, MEMO_SENT, name);
	}
}

/*************************************************************************/

static void do_cancel(User *u)
{
	char *name = strtok(NULL, " ");	
	NickAlias *na;
	
	if (!memos.writable()) {
		notice_lang(s_MemoServ, u, MEMO_CANCEL_DISABLED);
	} else if (!name) {
		syntax_error(s_MemoServ, u, "CANCEL", MEMO_CANCEL_SYNTAX);
	} else if (!nick_recognized(u)) {
		notice_lang(s_MemoServ, u, NICK_IDENTIFY_REQUIRED, s_NickServ);
	} else if (!(na = aliases.find(name))) {
		notice_lang(s_MemoServ, u, NICK_X_NOT_REGISTERED, name);
	} else {
		MemoInfo *mi = &na->ng()->memos;
		
		for (int i = mi->memos().size() - 1; i >= 0; i--) 
		{
			if ((mi->memos()[i]->flags() & Memo::mfUnread) && 
					(mi->memos()[i]->created() > u->na->ng()->created()) &&
					!stricmp(mi->memos()[i]->sender(), u->na->ng()->name()))
			{
				if (delmemo(name, mi, mi->memos()[i]->number())) {
					notice_lang(s_MemoServ, u, MEMO_CANCELLED, name);
				} else {
					notice_lang(s_MemoServ, u, MEMO_CANCEL_FAILED);
				}
					
				return;
			}
		}
		
		notice_lang(s_MemoServ, u, MEMO_CANCEL_NONE);
	}
}

/*************************************************************************/

/* Display a single memo entry, possibly printing the header first. */

static int list_memo(User *u, int index, MemoInfo *mi, int *sent_header,
			int newmemo)
{
	Memo *m;
	char timebuf[64];
	struct tm tm;
	time_t t;
	
	if (index < 0 || index >= mi->memos().size())
		return 0;
	if (!*sent_header) 
	{
	    notice_lang(s_MemoServ, u,
				newmemo ? MEMO_LIST_NEW_MEMOS : MEMO_LIST_MEMOS,
				u->nick, s_MemoServ);
		notice_lang(s_MemoServ, u, MEMO_LIST_HEADER);
		*sent_header = 1;
	}
	m = mi->memos()[index];
	t = m->created();
	tm = *localtime(&t);
	strftime_lang(timebuf, sizeof(timebuf),
			u, STRFTIME_DATE_TIME_FORMAT, &tm);
	timebuf[sizeof(timebuf)-1] = 0;	/* just in case */
	notice_lang(s_MemoServ, u, MEMO_LIST_FORMAT,
			(m->flags() & Memo::mfUnread) ? '*' : ' ',
			m->number(), m->sender(), timebuf);
	return 1;
}

static int list_memo_callback(User *u, int num, va_list args)
{
	MemoInfo *mi = va_arg(args, MemoInfo *);
	int *sent_header = va_arg(args, int *);
	int i;
	
	for (i = 0; i < mi->memos().size(); i++) 
		if (mi->memos()[i]->number() == (uint32_t)num)
			break;
	
	/* Range checking done by list_memo() */
	return list_memo(u, i, mi, sent_header, 0);
}


/* List the memos (if any) for the source nick or given channel. */

static void do_list(User *u)
{
	char *param = strtok(NULL, " ");
	MemoInfo *mi;
	int i;
	
	if (!nick_identified(u)) 
	{
		notice_lang(s_MemoServ, u, NICK_IDENTIFY_REQUIRED, s_NickServ);
		return;
	}
	mi = &u->na->ng()->memos;
	
	if (param && !isdigit(*param) && stricmp(param, "NEW") != 0) {
		syntax_error(s_MemoServ, u, "LIST", MEMO_LIST_SYNTAX);
	} else if (mi->memos().size() == 0) {
		notice_lang(s_MemoServ, u, MEMO_HAVE_NO_MEMOS);
	} else {
		int sent_header = 0;
		if (param && isdigit(*param)) {
			process_numlist(param, NULL, list_memo_callback, u,
					mi, &sent_header);
		} else {
			if (param) 
			{
				for (i = 0; i < mi->memos().size(); i++) 
				{
					if (mi->memos()[i]->flags() & Memo::mfUnread)
						break;
				}
				if (i == mi->memos().size()) 
				{
					notice_lang(s_MemoServ, u, MEMO_HAVE_NO_NEW_MEMOS);
				    return;
				}
			}
			
			for (i = 0; i < mi->memos().size(); i++) 
			{
				if (param && !(mi->memos()[i]->flags() & Memo::mfUnread))
					continue;
				list_memo(u, i, mi, &sent_header, param != NULL);
			}
		}
	}
}

/*************************************************************************/

/* Send a single memo to the given user. */

static int read_memo(User *u, int index, MemoInfo *mi)
{
	Memo *m;
	char timebuf[64];
	struct tm tm;
	time_t t;
	
	if (index < 0 || index >= mi->memos().size())
	return 0;
	m = mi->memos()[index];
	t = m->created();
	tm = *localtime(&t);
	strftime_lang(timebuf, sizeof(timebuf),
		u, STRFTIME_DATE_TIME_FORMAT, &tm);
	timebuf[sizeof(timebuf)-1] = 0;
	
	if (m->target())
		notice_lang(s_MemoServ, u, MEMO_HEADER_TARGET, m->number(),
					m->sender(), m->target(), timebuf, s_MemoServ,
					m->number());
	else
		notice_lang(s_MemoServ, u, MEMO_HEADER, m->number(),
					m->sender(), timebuf, s_MemoServ, m->number());
	notice_lang(s_MemoServ, u, MEMO_TEXT, m->text());
	m->flags_remove(Memo::mfUnread);
	return 1;
}

static int read_memo_callback(User *u, int num, va_list args)
{
	MemoInfo *mi = va_arg(args, MemoInfo *);
	int i;
	
	for (i = 0; i < mi->memos().size(); i++)
		if (mi->memos()[i]->number() == (uint32_t)num)
			break;
	
	/* Range check done in read_memo */
	return read_memo(u, i, mi);
}


/* Read memos. */

static void do_read(User *u)
{
	MemoInfo *mi;
	char *numstr = strtok(NULL, " ");
	int num, count;
	
	
	if (!nick_identified(u)) 
	{
		notice_lang(s_MemoServ, u, NICK_IDENTIFY_REQUIRED, s_NickServ);
		return;
	}
	mi = &u->na->ng()->memos;
	
	num = numstr ? atoi(numstr) : -1;
	if (!numstr || (stricmp(numstr,"LAST") != 0 && stricmp(numstr,"NEW") != 0
	                && num <= 0)) {
		syntax_error(s_MemoServ, u, "READ", MEMO_READ_SYNTAX);
	} else if (mi->memos().size() == 0) {
		notice_lang(s_MemoServ, u, MEMO_HAVE_NO_MEMOS);
	} else {
		int i;
		
		if (stricmp(numstr, "NEW") == 0) {
			int readcount = 0;
			for (i = 0; i < mi->memos().size(); i++) 
			{
				if (mi->memos()[i]->flags() & Memo::mfUnread) 
				{
					read_memo(u, i, mi);
					readcount++;
				}
			}
			if (!readcount)
				notice_lang(s_MemoServ, u, MEMO_HAVE_NO_NEW_MEMOS);
		} else if (stricmp(numstr, "LAST") == 0) {
			for (i = 0; i < mi->memos().size()-1; i++)
				;
			read_memo(u, i, mi);
		} else {	/* number[s] */
		    if (!process_numlist(numstr, &count, read_memo_callback, u, mi)) 
		    {
				if (count == 1)
					notice_lang(s_MemoServ, u, MEMO_DOES_NOT_EXIST, num);
				else
					notice_lang(s_MemoServ, u, MEMO_LIST_NOT_FOUND, numstr);
		    }
		}
	
	}
}

/*************************************************************************/

/* Delete a single memo from a MemoInfo. */

static int del_memo_callback(User *u, int num, va_list args)
{
    MemoInfo *mi = va_arg(args, MemoInfo *);
    int *last = va_arg(args, int *);
    int *last0 = va_arg(args, int *);
    char **end = va_arg(args, char **);
    int *left = va_arg(args, int *);
    char *target = va_arg(args, char *);

    if (delmemo(target, mi, num)) {
	if (num != (*last)+1) {
	    if (*last != -1) {
		int len;
		if (*last0 != *last)
		    len = snprintf(*end, *left, ",%d-%d", *last0, *last);
		else
		    len = snprintf(*end, *left, ",%d", *last);
		*end += len;
		*left -= len;
	    }
	    *last0 = num;
	}
	*last = num;
	return 1;
    } else {
	return 0;
    }
}


/* Delete memos. */

static void do_del(User *u)
{
	if (!memos.writable())
	{
		notice_lang(s_MemoServ, u, MEMO_DEL_DISABLED);
		return;
	}
	
    MemoInfo *mi;
    char *numstr = strtok(NULL, "");
    int last, last0;
    char buf[BUFSIZE], *end;
    int delcount, count, left;

	if (!nick_identified(u)) {
	    notice_lang(s_MemoServ, u, NICK_IDENTIFY_REQUIRED, s_NickServ);
	    return;
	}
	mi = &u->na->ng()->memos;
	
    if (!numstr || (!isdigit(*numstr) && stricmp(numstr, "ALL") != 0)) {
	syntax_error(s_MemoServ, u, "DEL", MEMO_DEL_SYNTAX);
    } else if (mi->memos().size() == 0) {
		notice_lang(s_MemoServ, u, MEMO_HAVE_NO_MEMOS);
    } else {
	if (isdigit(*numstr)) {
	    /* Delete a specific memo or memos. */
	    last = -1;   /* Last memo deleted */
	    last0 = -1;  /* Beginning of range of last memos deleted */
	    end = buf;
	    left = sizeof(buf);
	    delcount = process_numlist(numstr, &count, del_memo_callback, u, mi,
						&last, &last0, &end, &left, u->na->ng()->name());
	    if (last != -1) {
		/* Some memos got deleted; tell them which ones. */
		if (delcount > 1) {
		    if (last0 != last)
			end += snprintf(end, sizeof(buf)-(end-buf),
				",%d-%d", last0, last);
		    else
			end += snprintf(end, sizeof(buf)-(end-buf),
				",%d", last);
		    /* "buf+1" here because *buf == ',' */
		    notice_lang(s_MemoServ, u, MEMO_DELETED_SEVERAL, buf+1);
		} else {
		    notice_lang(s_MemoServ, u, MEMO_DELETED_ONE, last);
		}
	    } else {
		/* No memos were deleted.  Tell them so. */
		if (count == 1)
		    notice_lang(s_MemoServ, u, MEMO_DOES_NOT_EXIST,
				atoi(numstr));
		else
		    notice_lang(s_MemoServ, u, MEMO_DELETED_NONE);
	    }
	} else {
	    /* Delete all memos. */
		if (memos.remove_owned(&groups, u->na->ng(), 0))
			notice_lang(s_MemoServ, u, MEMO_DELETED_ALL);
		else
			notice_lang(s_MemoServ, u, MEMO_DEL_FAILED);
	}
	}
}

/*************************************************************************/

static void do_set(User *u)
{
	if (!groups.writable()) 
	{
		notice_lang(s_MemoServ, u, MEMO_SET_DISABLED);
		return;
    }
	
    char *cmd    = strtok(NULL, " ");
    char *param  = strtok(NULL, "");
    MemoInfo *mi = &u->na->ng()->memos;

    if (!param) {
	syntax_error(s_MemoServ, u, "SET", MEMO_SET_SYNTAX);
    } else if (!nick_identified(u)) {
	notice_lang(s_MemoServ, u, NICK_IDENTIFY_REQUIRED, s_NickServ);
	return;
    } else if (stricmp(cmd, "NOTIFY") == 0) {
	do_set_notify(u, mi, param);
    } else if (stricmp(cmd, "LIMIT") == 0) {
	do_set_limit(u, mi, param);
	} else if (stricmp(cmd, "FORWARD") == 0) {
	do_set_forward(u, mi, param);
    } else {
	notice_lang(s_MemoServ, u, MEMO_SET_UNKNOWN_OPTION, cmd);
	notice_lang(s_MemoServ, u, MORE_INFO, s_MemoServ, "SET");
    }
}

/*************************************************************************/

static void do_set_notify(User *u, MemoInfo *mi, char *param)
{
    if (stricmp(param, "ON") == 0) {
	u->na->ng()->flags_add(NickGroup::gfMemoSignon | NickGroup::gfMemoReceive);
	notice_lang(s_MemoServ, u, MEMO_SET_NOTIFY_ON, s_MemoServ);
    } else if (stricmp(param, "LOGON") == 0) {
	u->na->ng()->flags_add(NickGroup::gfMemoSignon);
	u->na->ng()->flags_remove(NickGroup::gfMemoReceive);
	notice_lang(s_MemoServ, u, MEMO_SET_NOTIFY_LOGON, s_MemoServ);
    } else if (stricmp(param, "NEW") == 0) {
	u->na->ng()->flags_add(NickGroup::gfMemoSignon);
	u->na->ng()->flags_remove(NickGroup::gfMemoReceive);
	notice_lang(s_MemoServ, u, MEMO_SET_NOTIFY_NEW, s_MemoServ);
    } else if (stricmp(param, "OFF") == 0) {
	u->na->ng()->flags_remove(NickGroup::gfMemoSignon | NickGroup::gfMemoReceive);
	notice_lang(s_MemoServ, u, MEMO_SET_NOTIFY_OFF, s_MemoServ);
    } else {
	syntax_error(s_MemoServ, u, "SET NOTIFY", MEMO_SET_NOTIFY_SYNTAX);
    }
}

/*************************************************************************/

static void do_set_limit(User *u, MemoInfo *mi, char *param)
{
    char *p1 = strtok(param, " ");
    char *p2 = strtok(NULL, " ");
    char *p3 = strtok(NULL, " ");
    char *user = NULL;
    int32_t limit;
    NickAlias *na = u->na;
    int is_servadmin = is_services_admin(u);

    if (is_servadmin) {
	if (p2 && stricmp(p2, "HARD") != 0) {
	    if (!(na = aliases.find(p1))) {
		notice_lang(s_MemoServ, u, NICK_X_NOT_REGISTERED, p1);
		return;
	    }
	    user = p1;
	    mi = &na->ng()->memos;
	    p1 = p2;
	    p2 = p3;
	} else if (!p1) {
	    syntax_error(s_MemoServ, u, "SET LIMIT",
					MEMO_SET_LIMIT_SERVADMIN_SYNTAX);
	    return;
	}
	if ((!isdigit(*p1) && stricmp(p1, "NONE") != 0) ||
			(p2 && stricmp(p2, "HARD") != 0)) {
	    syntax_error(s_MemoServ, u, "SET LIMIT",
					MEMO_SET_LIMIT_SERVADMIN_SYNTAX);
	    return;
	}
	
    if (p2)
	na->ng()->flags_add(NickGroup::gfMemoHardMax);
    else
	na->ng()->flags_remove(NickGroup::gfMemoHardMax);
			
	limit = atoi(p1);
	if (limit < 0 || limit > 32767) 
	{
	    notice_lang(s_MemoServ, u, MEMO_SET_LIMIT_OVERFLOW, 32767);
	    limit = 32767;
	}
	if (stricmp(p1, "NONE") == 0)
	    limit = -1;
    } else {
	if (!p1 || p2 || !isdigit(*p1)) {
	    syntax_error(s_MemoServ, u, "SET LIMIT", MEMO_SET_LIMIT_SYNTAX);
	    return;
	}
	if ((na->ng()->flags() & NickGroup::gfMemoHardMax)) 
	{
	    notice_lang(s_MemoServ, u, MEMO_SET_YOUR_LIMIT_FORBIDDEN);
	    return;
	}
	
	limit = atoi(p1);
	/* The first character is a digit, but we could still go negative
	 * from overflow... watch out! */
	if (limit < 0 || (epona->cd().memoserv.max_memos > 0 &&
			limit > epona->cd().memoserv.max_memos)) 
	{
		notice_lang(s_MemoServ, u, MEMO_SET_YOUR_LIMIT_TOO_HIGH,
			(uint16_t) epona->cd().memoserv.max_memos);
	    return;
	} else if (limit > 32767) {
	    notice_lang(s_MemoServ, u, MEMO_SET_LIMIT_OVERFLOW, 32767);
	    limit = 32767;
	}
    }
    mi->memomax(limit);
    if (limit > 0) {
	if (na->ng() == u->na->ng())
	    notice_lang(s_MemoServ, u, MEMO_SET_YOUR_LIMIT, limit);
	else
	    notice_lang(s_MemoServ, u, MEMO_SET_LIMIT, user, limit);
    } else if (limit == 0) {
	if (na->ng() == u->na->ng())
	    notice_lang(s_MemoServ, u, MEMO_SET_YOUR_LIMIT_ZERO);
	else
	    notice_lang(s_MemoServ, u, MEMO_SET_LIMIT_ZERO, user);
    } else {
	if (na->ng() == u->na->ng())
	    notice_lang(s_MemoServ, u, MEMO_UNSET_YOUR_LIMIT);
	else
	    notice_lang(s_MemoServ, u, MEMO_UNSET_LIMIT, user);
    }
}

/*************************************************************************/

static void do_set_forward(User *u, MemoInfo *mi, char *param)
{
	if (!epona->cd().memoserv.forward || !mail::Mail::configured()) {
		notice_lang(s_MemoServ, u, MEMO_SET_FORWARD_UNAVAILABLE);
	} else if (!u->na->ng()->email()) {
		notice_lang(s_MemoServ, u, MEMO_SET_FORWARD_NO_EMAIL, s_NickServ);
	} else if (!(u->na->ng()->flags() & NickGroup::gfAuth)) {
		notice_lang(s_MemoServ, u, MEMO_SET_FORWARD_NOT_AUTH, s_NickServ);
	} else if (stricmp(param, "ON") == 0) {
		u->na->ng()->flags_add(NickGroup::gfMemoForward);
		notice_lang(s_MemoServ, u, MEMO_SET_FORWARD_ON);
	} else if (stricmp(param, "OFF") == 0) {
		u->na->ng()->flags_remove(NickGroup::gfMemoForward);
		notice_lang(s_MemoServ, u, MEMO_SET_FORWARD_OFF);
	} else {
		syntax_error(s_MemoServ, u, "SET FORWARD", MEMO_SET_FORWARD_SYNTAX);
	}
}

/*************************************************************************/

static void do_info(User *u)
{
    MemoInfo *mi;
    NickAlias *na = NULL;
    char *name = strtok(NULL, " ");
    int is_servadmin = is_services_admin(u);
    int hardmax = 0;

    if (is_servadmin && name) {
	na = aliases.find(name);
	if (!na) {
	    notice_lang(s_MemoServ, u, NICK_X_NOT_REGISTERED, name);
	    return;
	}
	mi = &na->ng()->memos;
	hardmax = na->ng()->flags() & NickGroup::gfMemoHardMax ? 1 : 0;
	} else if (name) { /* We aren't services admin */
		notice_lang(s_MemoServ, u, ACCESS_DENIED);
		return;
    } else { /* !name */
	if (!nick_identified(u)) {
	    notice_lang(s_MemoServ, u, NICK_IDENTIFY_REQUIRED, s_NickServ);
	    return;
	}
	mi = &u->na->ng()->memos;
	hardmax = u->na->ng()->flags() & NickGroup::gfMemoHardMax ? 1 : 0;
    }

    if (name && na->ng() != u->na->ng()) {

		if (!mi->memos().size()) {
	    	notice_lang(s_MemoServ, u, MEMO_INFO_X_NO_MEMOS, name);
		} else if (mi->memos().size() == 1) {
	    	if (mi->memos()[0]->flags() & Memo::mfUnread)
			notice_lang(s_MemoServ, u, MEMO_INFO_X_MEMO_UNREAD, name);
	    	else
			notice_lang(s_MemoServ, u, MEMO_INFO_X_MEMO, name);
		} else {
	    	int count = 0, i;
	    	for (i = 0; i < mi->memos().size(); i++) {
				if (mi->memos()[i]->flags() & Memo::mfUnread)
		    		count++;
	    	}
	    	if (count == mi->memos().size())
				notice_lang(s_MemoServ, u, MEMO_INFO_X_MEMOS_ALL_UNREAD, name, count);
	    	else if (count == 0)
				notice_lang(s_MemoServ, u, MEMO_INFO_X_MEMOS, name, mi->memos().size());
	    	else if (count == 0)
				notice_lang(s_MemoServ, u, MEMO_INFO_X_MEMOS_ONE_UNREAD, name, mi->memos().size());
	    	else
				notice_lang(s_MemoServ, u, MEMO_INFO_X_MEMOS_SOME_UNREAD, name, mi->memos().size(), count);
		}
		if (mi->memomax() >= 0) {
	    	if (hardmax)
				notice_lang(s_MemoServ, u, MEMO_INFO_X_HARD_LIMIT, name, mi->memomax());
	    	else
				notice_lang(s_MemoServ, u, MEMO_INFO_X_LIMIT, name, mi->memomax());
		} else {
	    	notice_lang(s_MemoServ, u, MEMO_INFO_X_NO_LIMIT, name);
		}
	
		/* I ripped this code out of ircservices 4.4.5, since I didn't want
	   	   to rewrite the whole thing (it pisses me off). */
		if (na) {
    		if ((na->ng()->flags() & NickGroup::gfMemoReceive) && (na->ng()->flags() & NickGroup::gfMemoSignon)) {
        		notice_lang(s_MemoServ, u, MEMO_INFO_X_NOTIFY_ON, name);
        	} else if (na->ng()->flags() & NickGroup::gfMemoReceive) {
            	notice_lang(s_MemoServ, u, MEMO_INFO_X_NOTIFY_RECEIVE, name);
        	} else if (na->ng()->flags() & NickGroup::gfMemoSignon) {
            	notice_lang(s_MemoServ, u, MEMO_INFO_X_NOTIFY_SIGNON, name);
        	} else {
            	notice_lang(s_MemoServ, u, MEMO_INFO_X_NOTIFY_OFF, name);
        	}
    	}
    	
    	if (na)
    	{
    		if ((na->ng()->flags() & NickGroup::gfMemoForward) &&
    				mail::Mail::configured())
    			notice_lang(s_MemoServ, u, MEMO_INFO_X_FORWARD_ON, name);
    		else
    			notice_lang(s_MemoServ, u, MEMO_INFO_X_FORWARD_OFF, name);
    	}

    } else { /* !name || (!ci || na->ng() == u->na->ng()) */

		if (!mi->memos().size()) {
	    	notice_lang(s_MemoServ, u, MEMO_INFO_NO_MEMOS);
		} else if (mi->memos().size() == 1) {
	    	if (mi->memos()[0]->flags() & Memo::mfUnread)
				notice_lang(s_MemoServ, u, MEMO_INFO_MEMO_UNREAD);
	    	else
				notice_lang(s_MemoServ, u, MEMO_INFO_MEMO);
		} else {
	    	int count = 0, i;
	    	for (i = 0; i < mi->memos().size(); i++) {
				if (mi->memos()[i]->flags() & Memo::mfUnread)
		    		count++;
	    	}
	    	if (count == mi->memos().size())
				notice_lang(s_MemoServ, u, MEMO_INFO_MEMOS_ALL_UNREAD, count);
	    	else if (count == 0)
				notice_lang(s_MemoServ, u, MEMO_INFO_MEMOS, mi->memos().size());
	    	else if (count == 1)
				notice_lang(s_MemoServ, u, MEMO_INFO_MEMOS_ONE_UNREAD, mi->memos().size());
	    	else
				notice_lang(s_MemoServ, u, MEMO_INFO_MEMOS_SOME_UNREAD, mi->memos().size(), count);
		}
	
		if (mi->memomax() == 0) {
	    	if (!is_servadmin && hardmax)
				notice_lang(s_MemoServ, u, MEMO_INFO_HARD_LIMIT_ZERO);
	    	else
				notice_lang(s_MemoServ, u, MEMO_INFO_LIMIT_ZERO);
		} else if (mi->memomax() > 0) {
	    	if (!is_servadmin && hardmax)
				notice_lang(s_MemoServ, u, MEMO_INFO_HARD_LIMIT, mi->memomax());
	    	else
				notice_lang(s_MemoServ, u, MEMO_INFO_LIMIT, mi->memomax());
		} else {
	    	notice_lang(s_MemoServ, u, MEMO_INFO_NO_LIMIT);
		}
	
		/* Ripped too. But differently because of a seg fault (laughs) */
		if ((u->na->ng()->flags() & NickGroup::gfMemoReceive) &&
				(u->na->ng()->flags() & NickGroup::gfMemoSignon)) {
    		notice_lang(s_MemoServ, u, MEMO_INFO_NOTIFY_ON);
    	} else if (u->na->ng()->flags() & NickGroup::gfMemoReceive) {
    		notice_lang(s_MemoServ, u, MEMO_INFO_NOTIFY_RECEIVE);
    	} else if (u->na->ng()->flags() & NickGroup::gfMemoSignon) {
    		notice_lang(s_MemoServ, u, MEMO_INFO_NOTIFY_SIGNON);
    	} else {
       		notice_lang(s_MemoServ, u, MEMO_INFO_NOTIFY_OFF);
    	}
    	
    	if ((u->na->ng()->flags() & NickGroup::gfMemoForward) &&
    			mail::Mail::configured())
    		notice_lang(s_MemoServ, u, MEMO_INFO_FORWARD_ON);
    	else
    		notice_lang(s_MemoServ, u, MEMO_INFO_FORWARD_OFF);
    } /* if (name && (ci || na->ng() != u->na->ng())) */
}

/*************************************************************************/
