/* XMLexport interface.
 *
 * Epona (c) 2000-2002, 2004 PegSoft
 * Contact us at epona@pegsoft.net
 *
 * Based on the original code of Services by Andy Church.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file COPYING); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef XMLEXPORT_H
#define XMLEXPORT_H

/*************************************************************************/

#include <databases/xml.h>

/*************************************************************************/
	
namespace legacydb 
{

/*************************************************************************/

class XMLExport : public databases::XMLExport
{
	public:
		virtual bool export_data(std::string const &dir,
				databases::XMLIndex &index, std::string const &database = "");
	private:
		bool version(databases::XMLWriter &xw, uint16_t version, 
				char const *filename);
		bool add_field(databases::XMLWriter &xw, char const *name,
				uint16_t type, size_t size, uint16_t attrs, char const *defval,
				char const *filename);
		bool count(databases::XMLWriter &xw, uint32_t count,
				char const *filename);
		bool begin_record(databases::XMLWriter &xw, char const *filename);
		bool add_value(databases::XMLWriter &xw, char const *name,
				char const *value, char const *filename);
		bool add_ivalue(databases::XMLWriter &xw, char const *name,
				int32_t value, char const *filename);
		bool add_rivalue(databases::XMLWriter &xw, char const *name,
				databases::Record::identifier_type value, char const *filename);
		bool add_uivalue(databases::XMLWriter &xw, char const *name,
				uint32_t value, char const *filename);
		bool add_utvalue(databases::XMLWriter &xw, char const *name,
				time_t value, char const *filename);
		bool end_record(databases::XMLWriter &xw, char const *filename);
		bool save(databases::XMLWriter &xw, std::string const &filename);
	
		bool add_index(databases::XMLIndex &xi, std::string const &filename);
	
		bool export_nickserv(std::string const &dir,
				databases::XMLIndex &xi, std::string const &database);
		bool export_botserv(std::string const &dir,
				databases::XMLIndex &xi, std::string const &database);
		bool export_chanserv(std::string const &dir,
				databases::XMLIndex &xi, std::string const &database);
		bool export_operserv(std::string const &dir,
				databases::XMLIndex &xi, std::string const &database);
		bool export_news(std::string const &dir,
				databases::XMLIndex &xi, std::string const &database);
		bool export_exceptions(std::string const &dir,
				databases::XMLIndex &xi, std::string const &database);
};

/*************************************************************************/

#define NICKMAX 32
#define PASSMAX 32
#define CHANMAX 64

/* Nickname status flags: */
#define NS_PENDING_EXPIRE	0x0001		/* Expiration pending */
#define NS_VERBOTEN			0x0002  	/* Nick may not be registered or used */
#define NS_NO_EXPIRE		0x0004      /* Nick never expires */
#define NS_UNUSED           0xFFFA      /* Unused flags */

/* Nickname setting flags: */
#define NI_KILLPROTECT		0x00000001  /* Kill others who take this nick */
#define NI_SECURE			0x00000002  /* Don't recognize unless IDENTIFY'd */
#define NI_MEMO_HARDMAX		0x00000008  /* Don't allow user to change memo limit */
#define NI_MEMO_SIGNON		0x00000010  /* Notify of memos at signon and un-away */
#define NI_MEMO_RECEIVE		0x00000020  /* Notify of new memos when sent */
#define NI_PRIVATE			0x00000040  /* Don't show in LIST to non-servadmins */
#define NI_HIDE_EMAIL		0x00000080  /* Don't show E-mail in INFO */
#define NI_HIDE_MASK		0x00000100  /* Don't show last seen address in INFO */
#define NI_HIDE_QUIT		0x00000200  /* Don't show last quit message in INFO */
#define NI_KILL_QUICK		0x00000400  /* Kill in 20 seconds instead of 60 */
#define NI_KILL_IMMED		0x00000800  /* Kill immediately instead of in 60 sec */
#define NI_SERVICES_OPER 	0x00001000	/* User is a Services operator */
#define NI_SERVICES_ADMIN	0x00002000	/* User is a Services admin */
#define NI_ENCRYPTEDPW		0x00004000	/* Nickname password is encrypted */
#define NI_SERVICES_ROOT    0x00008000  /* User is a Services root */
#define NI_CUSTOM_HOST      0x00010000  /* Enable custom hostname */

#define NI_UNUSED           0xFFFE4004  /* Unused bits */

/*************************************************************************/

/* Note that these two levels also serve as exclusive boundaries for valid
 * access levels.  ACCESS_FOUNDER may be assumed to be strictly greater
 * than any valid access level, and ACCESS_INVALID may be assumed to be
 * strictly less than any valid access level.
 */
#define ACCESS_FOUNDER	10000	/* Numeric level indicating founder access */
#define ACCESS_INVALID	-10000	/* Used in levels[] for disabled settings */

/* Levels for xOP */

#define ACCESS_VOP		3
#ifdef HAS_HALFOP
# define ACCESS_HOP     4
#endif
#define ACCESS_AOP		5
#define ACCESS_SOP		10


/* AutoKick data. */

#define AK_USED		0x0001
#define AK_ISNICK	0x0002
#define AK_STUCK    0x0004

#define AK_UNUSED   0xFFF9

/* Badword data */

#define BW_ANY 		0
#define BW_SINGLE 	1
#define BW_START 	2
#define BW_END 		3

#define CA_SIZE		36
#define KBB_SIZE 	8

/* Retain topic even after last person leaves channel */
#define CI_KEEPTOPIC	0x00000001
/* Don't allow non-authorized users to be opped */
#define CI_SECUREOPS	0x00000002
/* Hide channel from ChanServ LIST command */
#define CI_PRIVATE	0x00000004
/* Topic can only be changed by SET TOPIC */
#define CI_TOPICLOCK	0x00000008
/* Those not allowed ops are kickbanned */
#define CI_RESTRICTED	0x00000010
/* Don't allow ChanServ and BotServ commands to do bad things to bigger levels */
#define CI_PEACE  0x00000020
/* Don't allow any privileges unless a user is IDENTIFY'd with NickServ */
#define CI_SECURE	0x00000040
/* Don't allow the channel to be registered or used */
#define CI_VERBOTEN	0x00000080
/* Channel password is encrypted */
#define CI_ENCRYPTEDPW	0x00000100
/* Channel does not expire */
#define CI_NO_EXPIRE	0x00000200
/* Channel memo limit may not be changed */
#define CI_MEMO_HARDMAX	0x00000400
/* Send notice to channel on use of OP/DEOP */
#define CI_OPNOTICE	0x00000800
/* Stricter control of channel founder status */
#define CI_SECUREFOUNDER 0x00001000
/* Always sign kicks */
#define CI_SIGNKICK 0x00002000
/* Sign kicks if level is < than the one defined by the SIGNKICK level */
#define CI_SIGNKICK_LEVEL 0x00004000
/* Use the xOP lists */
#define CI_XOP 0x00008000
/* Pending expiration */
#define CI_PENDING_EXPIRE 0x00010000

/* Default flags */
#define CI_DEFAULT (CI_KEEPTOPIC | CI_PEACE | CI_SECURE | CI_SECUREFOUNDER | CI_SIGNKICK)

/* TEMPORARY - ChanServ is on the channel. */
#define CI_INHABIT 0x80000000

/* Unused flags */
#define CI_UNUSED 0x3FFE8580
/* Temporary flags */
#define CI_TEMPORARY 0x80000000

/* Indices for cmd_access[]: */
#define CA_INVITE			0
#define CA_AKICK			1
#define CA_SET				2	/* but not FOUNDER or PASSWORD */
#define CA_UNBAN			3
#define CA_AUTOOP			4
#define CA_AUTODEOP			5	/* Maximum, not minimum */
#define CA_AUTOVOICE		6
#define CA_OPDEOP			7	/* ChanServ commands OP and DEOP */
#define CA_ACCESS_LIST		8
#define CA_CLEAR			9
#define CA_NOJOIN			10	/* Maximum */
#define CA_ACCESS_CHANGE 	11
#define CA_MEMO				12
#define CA_ASSIGN       	13  /* BotServ ASSIGN command */
#define CA_BADWORDS     	14  /* BotServ BADWORDS command */
#define CA_NOKICK       	15  /* Not kicked by the bot */
#define CA_FANTASIA   		16
#define CA_SAY				17
#define CA_GREET            18
#define CA_VOICEME			19
#define CA_VOICE			20
#define CA_GETKEY           21
#define CA_AUTOHALFOP		22
#define CA_AUTOPROTECT		23
#define CA_OPDEOPME         24
#define CA_HALFOPME         25
#define CA_HALFOP           26
#define CA_PROTECTME        27
#define CA_PROTECT          28
#define CA_KICKME           29
#define CA_KICK             30
#define CA_SIGNKICK			31
#define CA_BANME            32
#define CA_BAN              33
#define CA_TOPIC            34
#define CA_INFO             35

/* Modes that have been removed. */
#define REMOVED_CMODE_H 0x00001000

/* BotServ SET flags */
#define BS_DONTKICKOPS      0x00000001
#define BS_DONTKICKVOICES   0x00000002
#define BS_FANTASY          0x00000004
#define BS_SYMBIOSIS        0x00000008
#define BS_GREET            0x00000010
#define BS_NOBOT            0x00000020

#define BS_DEFAULT	(BS_DONTKICKOPS | BS_FANTASY | BS_SYMBIOSIS)

/* BotServ Kickers flags */
#define BS_KICK_BOLDS 		0x80000000
#define BS_KICK_COLORS 		0x40000000
#define BS_KICK_REVERSES	0x20000000
#define BS_KICK_UNDERLINES	0x10000000
#define BS_KICK_BADWORDS	0x08000000
#define BS_KICK_CAPS		0x04000000
#define BS_KICK_FLOOD		0x02000000
#define BS_KICK_REPEAT		0x01000000

#define BS_UNUSED           0x00FFFF80

/* Indices for KBB (Kicks Before Ban) */
#define KBB_BOLDS 		0
#define KBB_COLORS 		1
#define KBB_REVERSES	2
#define KBB_UNDERLINES  3
#define KBB_BADWORDS	4
#define KBB_CAPS		5
#define KBB_FLOOD		6
#define KBB_REPEAT		7

/*************************************************************************/

class DefEntry
{
	public:
		/// Constructor.
		DefEntry(int index, char const *name, int defval) :
				m_index(index), m_name(name), m_defval(defval) { }
		/// Copy constructor.
		DefEntry(DefEntry const &de) : m_index(de.m_index), m_name(de.m_name),
				m_defval(de.m_defval) { }
		/// Assignment operator.
		DefEntry &operator=(DefEntry const &right);
		
		/// Returns the index.
		int index() const { return m_index; }
		/// Returns the name.
		char const *name() const { return m_name.c_str(); }
		/// Returns the default value.
		int defval() const { return m_defval; }
	private:
		int m_index;
		std::string m_name;
		int m_defval;
};

/*************************************************************************/

struct CBModeInfo
{
	char mode;				/* The mode (CMODE_* below) */
	uint32_t flag;			/* Long value that represents the mode */
	uint16_t flags;			/* CBM_* below */
	
	/* Parameter index (-1 == none). */
	int pidx;
};

#define CMODE_i 0x00000001
#define CMODE_m 0x00000002
#define CMODE_n 0x00000004
#define CMODE_p 0x00000008
#define CMODE_s 0x00000010
#define CMODE_t 0x00000020
#define CMODE_k 0x00000040		/* These two used only by ChanServ */
#define CMODE_l 0x00000080

#define CMODE_R 0x00000100		/* Only identified users can join */
#define CMODE_r 0x00000200		/* Set for all registered channels */

/* This mode is for IRC_BAHAMUT servers only. */
#define CMODE_BAHAMUT_c 0x00000400		/* Colors can't be used */
#define CMODE_BAHAMUT_M 0x00000800      /* Non-regged nicks can't send messages */
#define CMODE_BAHAMUT_j 0x00001000      /* Join rate limit */
#define CMODE_BAHAMUT_O 0x00008000		/* Only opers can join */

/* These modes are for IRC_ULTIMATE servers only. */
#define CMODE_ULTIMATE_f 0x00000400
#define CMODE_ULTIMATE_x 0x00000800
#define CMODE_ULTIMATE_A 0x00001000
#define CMODE_ULTIMATE_I 0x00002000
#define CMODE_ULTIMATE_K 0x00004000
#define CMODE_ULTIMATE_L 0x00008000
#define CMODE_ULTIMATE_O 0x00010000
#define CMODE_ULTIMATE_S 0x00020000

/* These modes are for IRC_UNREAL servers only. */
#define CMODE_UNREAL_c 0x00000400
#define CMODE_UNREAL_A 0x00000800
#define CMODE_UNREAL_j 0x00001000
#define CMODE_UNREAL_K 0x00002000
#define CMODE_UNREAL_L 0x00004000
#define CMODE_UNREAL_O 0x00008000
#define CMODE_UNREAL_Q 0x00010000
#define CMODE_UNREAL_S 0x00020000
#define CMODE_UNREAL_V 0x00040000
#define CMODE_UNREAL_f 0x00080000
#define CMODE_UNREAL_G 0x00100000
#define CMODE_UNREAL_C 0x00200000
#define CMODE_UNREAL_u 0x00400000
#define CMODE_UNREAL_z 0x00800000
#define CMODE_UNREAL_N 0x01000000
#define CMODE_UNREAL_M 0x02000000
#define CMODE_UNREAL_T 0x04000000

#define CBM_MINUS_NO_ARG 	0x0001		/* No argument for unset */

/*************************************************************************/

} /* namespace legacydb */

/*************************************************************************/

#endif /* XMLEXPORT_H */
