/* MySQL module.
 *
 * MySQL database module (c) 2005 PegSoft
 * Contact us at pegsoft@pegsoft.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file COPYING); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef MODULE_H
#define MODULE_H

/*************************************************************************/

#include <modules/module.h>
#include <scheduler/scheduler.h>

#include <mysql-cli/conn.h>

/*************************************************************************/

namespace modules { class Core; }

/*************************************************************************/

/// Namespace for module mysql.

namespace mysql 
{

/*************************************************************************/

class DBIndex;
class Directives;

/*************************************************************************/

/// The module version.
static int const mysql_version = 0x01000000;

/// The mysql module class.

class MySQL : public modules::Module
{
	public:
		/// The reconnect task.
		class RecoTask : public scheduler::TaskTimeout
		{
			public:
				/// Constructor.
				RecoTask() : scheduler::TaskTimeout(-1, 0) { }
				
				/// Runs the task.
				virtual int run();
				
				/// Overrides scheduler::Task::description().
				virtual std::string description() const
						{ return "MySQL reconnection attempt task"; }
				/// Overrides scheduler::Task::name().
				virtual std::string name() const { return "mysql-reconnect"; }
		};
		
		/// The sync task.
		class SyncTask : public scheduler::TaskTimeout
		{
			public:
				/// Constructor.
				SyncTask() : scheduler::TaskTimeout(-1, 0) { }
				
				/// Runs the task.
				virtual int run();
				
				/// Overrides scheduler::Task::description().
				virtual std::string description() const
						{ return "MySQL sync task"; }
				/// Overrides scheduler::Task::name().
				virtual std::string name() const { return "mysql-sync"; }
		};
	
		/// Constructor.
		MySQL(modules::Core &core);
		/// Destructor.
		~MySQL();
		
		/// Overrides Module::config().
		virtual conf::ConfQ *config(modules::Core &core,
				conf::ConfGlobalQ &global);
		/// Overrides Module::factory().
		virtual modules::Interface *factory(modules::PSIID iid);
		/// Overrides Module::load().
		virtual bool load(modules::Core &core);
		/// Overrides Module::notify().
		virtual void notify(modules::Module &sender, int msg, void *arg);
		/// Overrides Module::query().
		virtual bool query(std::string const &query, std::string &feedback);
		/// Overrides Module::start().
		virtual void start(modules::Core &core);
		/// Overrides Module::stop().
		virtual bool stop(modules::Core &core);
		/// Overrides Module::unload().
		virtual void unload(modules::Core &core);
		/// Implements Module::version().
		virtual int version() const { return mysql_version; }
		
		/// Returns the core module.
		modules::Core &core() { return *m_core; }
		
		/// Returns the configuration directives.
		Directives const &cd() const { return *m_cd; }
		
		/// Returns the MySQL connection.
		mysql_cli::Conn &conn() { return *m_conn; }
		
		/// Returns the database index.
		DBIndex &index() { return *m_index; }
		
		/// Returns the reconnect task.
		RecoTask &recotask() { return m_recotask; }
		/// Returns the sync task.
		SyncTask &synctask() { return m_synctask; }
	private:
		modules::Core *m_core;
		Directives *m_cd;
		
		mysql_cli::Conn *m_conn;
		DBIndex *m_index;
		
		int m_stype;
		RecoTask m_recotask;
		SyncTask m_synctask;
};

/*************************************************************************/

} /* namespace mysql */

/*************************************************************************/

#endif /* MODULE_H */
