/* Sendmail mail transport module.
 *
 * Sendmail mail transport module (c) 2007 PegSoft
 * Contact us at pegsoft@pegsoft.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file COPYING); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <system/headers.h>

#include <core/version.h>
#include <log/levels.h>
#include <modules/core.h>

#include "conf.h"
#include "decl.h"
#include "module.h"

using namespace std;
using namespace modules;
using namespace sendmail;

/*************************************************************************/

extern "C" Module *sendmail_get(Core &core)
{
	/* Ensure the core version is the same as the version we were
	 * compiled for so that there are no compatibility problems later. */
	if (core.version() != core::core_version)
	{
		core.log(xlog::LogCritical(), "Core module version mismatch");
		return 0;
	}
	
	return (me = new Sendmail(core));
}

/*************************************************************************/

extern "C" void sendmail_release(Core &core, Module *module)
{
	delete module;
	me = 0;
}

/*************************************************************************/

namespace sendmail
{

/*************************************************************************/

Sendmail *me;

/*************************************************************************/

Sendmail::Sendmail(Core &core) : 
		Module("sendmail", "Sendmail-based mail transport module"),
		m_core(&core), m_cd(new Directives(core.conf_logger())), transport()
{
	
}

/*************************************************************************/

Sendmail::~Sendmail()
{
	delete m_cd;
}

/*************************************************************************/

conf::ConfQ *Sendmail::config(modules::Core &core, conf::ConfGlobalQ &global)
{
	return &m_cd->parser();
}

/*************************************************************************/

modules::Interface *Sendmail::interface(PSIID iid)
{
	if (iid == mail::Transport::IID)
		return &transport;
	else
		return Module::interface(iid);
}

/*************************************************************************/

} /* namespace sendmail */

/*************************************************************************/
