/* SHA1 module.
 *
 * SHA1 module (c) 2006 PegSoft
 * Contact us at pegsoft@pegsoft.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file COPYING); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <system/headers.h>

#include <core/version.h>
#include <log/levels.h>
#include <modules/core.h>

#include "decl.h"
#include "module.h"

using namespace std;
using namespace modules;
using namespace sha1;

/*************************************************************************/

extern "C" Module *sha1_get(Core &core)
{
	/* Ensure the core version is the same as the version we were
	 * compiled for so that there are no compatibility problems later. */
	if (core.version() != core::core_version)
	{
		core.log(xlog::LogCritical(), "Core module version mismatch");
		return 0;
	}
	
	return (me = new SHA1(core));
}

/*************************************************************************/

extern "C" void sha1_release(Core &core, Module *module)
{
	delete module;
	me = 0;
}

/*************************************************************************/

namespace sha1
{

/*************************************************************************/

SHA1 *me;

/*************************************************************************/

SHA1::SHA1(Core &core) : Module("sha1", "SHA1 encryption module"),
		m_core(&core), m_encrypter()
{
	
}

/*************************************************************************/

Interface *SHA1::interface(modules::PSIID iid)
{
	if (iid == password::Encrypter::IID)
		return &m_encrypter;
	
	return Module::interface(iid);
}
	
/*************************************************************************/

} /* namespace sha1 */

/*************************************************************************/
