/* Password interface.
 *
 * SHA1 module (c) 2006 PegSoft
 * Contact us at pegsoft@pegsoft.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file COPYING); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <system/headers.h>

#include <openssl/evp.h>

#include "decl.h"
#include "module.h"
#include "password.h"

using namespace std;

/*************************************************************************/

namespace sha1
{

/*************************************************************************/

bool Encrypter::check(char const *plain, char const *pass)
{
	char eplain[encpass_size + 1];
	
	if (!encrypt(plain, eplain, sizeof(eplain)))
		return false;
	return (strcmp(eplain, pass) == 0);
}

/*************************************************************************/

bool Encrypter::encrypt(char const *pass, char *buf, size_t bufsize)
{
	if (bufsize < length() + 1)
		return 0;
	
	unsigned int md_len;
	unsigned char md_value[EVP_MAX_MD_SIZE];

	EVP_MD_CTX md_ctx;
	EVP_MD const *md = EVP_sha1();

	EVP_MD_CTX_init(&md_ctx);
	if (!EVP_DigestInit_ex(&md_ctx, md, 0))
		return false;
	if (!EVP_DigestUpdate(&md_ctx, pass, strlen(pass)))
		return false;
	if (!EVP_DigestFinal_ex(&md_ctx, md_value, &md_len))
		return false;
	EVP_MD_CTX_cleanup(&md_ctx);

	char *end = buf;

	for (unsigned int i = 0; i < md_len; ++i)
		end += sprintf(end, "%02x", md_value[i]);

	buf[length()] = 0;
	return true;
}

/*************************************************************************/

bool Encrypter::encrypt(char *buf, size_t size)
{
	return encrypt(buf, buf, size);
}

/*************************************************************************/

modules::Module *Encrypter::module()
{
	return me;
}

/*************************************************************************/

} /* namespace sha1 */

/*************************************************************************/
