/* SMTP mail transport module.
 *
 * SMTP mail transport module (c) 2007 PegSoft
 * Contact us at pegsoft@pegsoft.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file COPYING); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef MODULE_H
#define MODULE_H

/*************************************************************************/

#include <list>

#include <modules/module.h>

#include "transport.h"

/*************************************************************************/

namespace modules { class Core; }

/*************************************************************************/

/// Namespace for module smtp_cli.

namespace smtp_cli 
{

/*************************************************************************/

class Directives;
class SMTPSocket;

/*************************************************************************/

/// The module version.
static int const smtp_cli_version = 0x01000000;

/// The smtp_cli module class.

class SMTPCli : public modules::Module
{
	public:
		/// Constructor.
		SMTPCli(modules::Core &core);
		/// Destructor.
		~SMTPCli();
		
		/// Overrides Module::config().
		virtual conf::ConfQ *config(modules::Core &core,
				conf::ConfGlobalQ &global);
		
		/// Overrides Module::interface().
		virtual modules::Interface *interface(modules::PSIID iid);
		
		/// Overrides Module::notify().
		virtual void notify(modules::Module &sender, int msg, void *arg);
		
		/// Overrides Module::stop().
		virtual bool stop(modules::Core &core);
		
		/// Implements Module::version().
		virtual int version() const { return smtp_cli_version; }
		
		/// Returns the core module.
		modules::Core &core() { return *m_core; }
		
		/// Returns the configuration directives.
		Directives const &cd() const { return *m_cd; }
		
		/// Adds a sock.
		void add_sock(SMTPSocket *sock) { m_socks.push_back(sock); }
		/// Removes a sock.
		void remove_sock(SMTPSocket *sock) { m_socks.remove(sock); }
		/// Returns the number of socks.
		size_t socks_size() const { return m_socks.size(); }
	private:
		modules::Core *m_core;
		Directives *m_cd;
		
		typedef std::list<SMTPSocket *> socks_list;
		socks_list m_socks;
		
		Transport m_transport;
};

/*************************************************************************/

} /* namespace smtp_cli */

/*************************************************************************/

#endif /* MODULE_H */
