/* Configuration.
 *
 * Twilight database module (c) 2005 PegSoft
 * Contact us at pegsoft@pegsoft.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file COPYING); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <system/headers.h>

#include <misc/misc.h>

#include "conf.h"
#include "decl.h"
#include "module.h"

using namespace std;
using namespace conf;

/*************************************************************************/

namespace twilight
{

/*************************************************************************/

#define CHECK_ADD(x, y, z) if (!(x).add((z))) { \
	(y).errorf("Can't add directive %s: out of memory?", (z).name().c_str()); \
	return false; }

/*************************************************************************/

Directives::Directives(conf::ConfLogger &logger) : DirectivesQ(logger),
		directory(), save_frequency()
{
	
}

/*************************************************************************/

bool Directives::add(conf::Conf &conf)
{
	CHECK_ADD(conf, conf, directory);
	CHECK_ADD(conf, conf, save_frequency);
	
	return true;
}

/*************************************************************************/

Directives::Directory::Directory() :
		ConfString("directory", PROGRAM_DIR "/databases", true, 1, PATH_MAX, "")
{
		
}

/*************************************************************************/

bool Directives::Directory::apply(conf::Conf &conf)
{
	if (!misc::recursive_mkdir(m_value.c_str(), 0777))
	{
		conf.errorf("Unable to create directory %s: %s", m_value.c_str(),
				m_value.c_str());
		return false;
	}
	
	return ConfString::apply(conf);
}

/*************************************************************************/

Directives::SaveFrequency::SaveFrequency() :
		ConfDuration("save-frequency", 300, true, 30, 86400)
{
	
}

/*************************************************************************/

bool Directives::SaveFrequency::apply_change(conf::Conf &conf)
{
	me->task().step(m_value);
	return ConfDuration::apply_change(conf);
}

/*************************************************************************/

} /* namespace twilight */

/*************************************************************************/
