/* Main header for Services.
 *
 * Epona (c) 2000-2002, 2004 PegSoft
 * Contact us at epona@pegsoft.net
 *
 * This program is free but copyrighted software; see the file COPYING for
 * details.
 *
 * Based on the original code of Services by Andy Church.
 *
 */

#ifndef SERVICES_H
#define SERVICES_H

/*************************************************************************/

#include <system/headers.h>

#if HAVE_NETINET_IN_H
# include <netinet/in.h>
#endif

#include "tweak.h"

/*************************************************************************/

typedef struct server_ Server;
typedef struct user_ User;
typedef struct channel_ Channel;
class ChannelInfo;
class NickAlias;
class NickGroup;

/*************************************************************************/

/* Miscellaneous definitions. */
#include "defs.h"

/*************************************************************************/
/*************************************************************************/

/* Protocol tweaks */

#ifdef IRC_BAHAMUT
# define HAS_CIDR4_AKILL
# define HAS_EXEMPTS        /* Has +e chan mode */
# define HAS_INVITES        /* Has +I chan mode */
# define HAS_IP_AKILL
# define HAS_JMODE          /* Has +j chan mode */
# define HAS_NICKIP
# define HAS_NOQUIT
# define HAS_REALNAME_AKILL
# define HAS_SERVER_IP_AKILL
# define HAS_SVSMODE_UNBAN
#endif

#ifdef IRC_SEQUANA
# define HAS_CHGHOST
# undef  HAS_EXEMPTS
# undef  HAS_INVITES
# define HAS_NICKVHOST
# define HAS_VHOST
# define HAS_VHOST_AKILL
#endif

#ifdef IRC_SEQUANA_LIGHT
# define HAS_CHGHOST
#endif

#ifdef IRC_ULTIMATE
# define HAS_CHGHOST
# define HAS_EXEMPTS
# define HAS_FMODE			/* Has +f chan mode */
# define HAS_HALFOP
# define HAS_LMODE			/* Has +L chan mode */
# define HAS_VHOST
#endif

#ifdef IRC_UNREAL
# define HAS_CHGHOST
# define HAS_CIDR4_AKILL
# define HAS_EXEMPTS
# define HAS_FMODE			/* Has +f chan mode */
# define HAS_HALFOP
# define HAS_INVITES        /* Has +I chan mode */
# define HAS_IP_AKILL
# define HAS_JMODE          /* Has +j chan mode */
# define HAS_LMODE			/* Has +L chan mode */
# define HAS_NICKIP
# define HAS_NICKVHOST
# define HAS_REALNAME_AKILL
# define HAS_SERVER_IP_AKILL
# define HAS_SVSMODE_UNBAN
# define HAS_VHOST
#endif

/* The ircd name */

#if defined(IRC_BAHAMUT)
# define IRCD_NAME "bahamut"
#elif defined(IRC_ULTIMATE)
# define IRCD_NAME "ultimate"
#elif defined(IRC_UNREAL)
# define IRCD_NAME "unreal"
#elif defined(IRC_DREAMFORGE)
# define IRCD_NAME "dreamforge"
#endif

/*************************************************************************/

/* Server data. */

struct server_ {
	Server *prev, *next;
	
	char name[SERVERMAX];
	
	int hopcount;
	char desc[SDESCMAX];
	
	Server *hub;
	int leafcount;		/* Number of leaf servers */
};

/*************************************************************************/

/* Online user and channel data. */

struct user_ {
    User *next, *prev;
    
    char nick[NICKMAX];
    
    char *username;
    char *host;             /* User's real hostname */
#ifdef HAS_VHOST
    char *vhost;            /* User's virtual hostname */
#endif
#ifdef HAS_CHGHOST
    char *ivhost;           /* User's initial virtual hostname */
#endif
    char *realname;
    Server *server;			/* Server user is on */
#ifdef HAS_NICKIP
    struct in_addr ip;		/* IP address */
#endif
    
    time_t timestamp;		/* Timestamp of the nick */
    time_t my_signon;		/* When did _we_ see the user? */
    uint32_t svid;			/* Services ID */ 
    uint32_t mode;			/* See below */
    
    NickAlias *na;
    
    struct u_chanlist {
		struct u_chanlist *next, *prev;
		Channel *chan;
		int16_t status;		/* Associated flags; see CSTATUS_* below. */
    } *chans;				/* Channels user has joined */
    
    struct u_chaninfolist {
		struct u_chaninfolist *next, *prev;
		ChannelInfo *chan;
    } *founder_chans;			/* Channels user has identified for */
    
    short invalid_pw_count;		/* # of invalid password attempts */
    time_t invalid_pw_time;		/* Time of last invalid password */
    
    time_t lastmemosend;		/* Last time MS SEND command used */
    time_t lastnickreg;			/* Last time NS REGISTER cmd used */
    time_t lastmail;			/* Last time this user sent a mail */
};

#define UMODE_a 0x00000001
#define UMODE_h 0x00000002
#define UMODE_i 0x00000004
#define UMODE_o 0x00000008
#define UMODE_r 0x00000010
#define UMODE_w 0x00000020
#define UMODE_A 0x00000040

#ifdef IRC_DREAMFORGE
# define UMODE_g 0x80000000
#endif

#ifdef IRC_BAHAMUT
# define UMODE_R 0x80000000
# define UMODE_I 0x20000000
#endif

#if defined(IRC_SEQUANA)
# define UMODE_H 0x40000000
#endif

#if defined(IRC_ULTIMATE) || defined(IRC_UNREAL)
# define UMODE_x 0x40000000
#endif

/* Returns *current* user hostname */
#if defined(IRC_SEQUANA)
# define GetHost(x)      ((x)->mode & UMODE_H ? (x)->vhost : (x)->host)
# define IsHostHidden(x) ((x)->mode & UMODE_H)
# define HiddenHostMode  "H"
#elif defined(IRC_ULTIMATE) || defined(IRC_UNREAL)
# define GetHost(x)      ((x)->mode & UMODE_x ? (x)->vhost : (x)->host)
# define IsHostHidden(x) ((x)->mode & UMODE_x)
# define HiddenHostMode  "x"
#else
# define GetHost(x)      ((x)->host)
# define IsHostHidden    (0)
#endif

#if defined(IRC_UNREAL)
# define BOT_MODES "+qS"
#elif defined(IRC_ULTIMATE)
# define BOT_MODES "+pS"
#else
# define BOT_MODES "+"
#endif

struct BanData;
struct UserData;

struct channel_ {
    Channel *next, *prev;
    char name[CHANMAX];
    ChannelInfo *ci;			/* Corresponding ChannelInfo */
    time_t creation_time;		/* When channel was created */

    char *topic;
    char topic_setter[NICKMAX];		/* Who set the topic */
    time_t topic_time;			/* When topic was set */

    uint32_t mode;				/* Binary modes only */
    int32_t limit;				/* 0 if none */
    char *key;				/* NULL if none */
#ifdef HAS_FMODE
	char *flood;			/* +f; NULL if none */
#endif
#ifdef HAS_JMODE
	char *joinrate;			/* +j; NULL if none */
#endif
#ifdef HAS_LMODE
	char *redirect;			/* +L; NULL if none */
#endif

    int16_t bancount, bansize;
    char **bans;

#ifdef HAS_EXEMPTS
	int16_t exemptcount, exemptsize;
	char **exempts;
#endif

    struct c_userlist {
		struct c_userlist *next, *prev;
		User *user;
		UserData *ud;
    } *users;
    int32_t usercount;

    BanData *bd;

    time_t server_modetime;		/* Time of last server MODE */
    time_t chanserv_modetime;	/* Time of last check_modes() */
    int16_t server_modecount;		/* Number of server MODEs this second */
    int16_t chanserv_modecount;	/* Number of check_mode()'s this sec */
    int16_t bouncy_modes;			/* Did we fail to set modes here? */
};

#define CMODE_i 0x00000001
#define CMODE_m 0x00000002
#define CMODE_n 0x00000004
#define CMODE_p 0x00000008
#define CMODE_s 0x00000010
#define CMODE_t 0x00000020
#define CMODE_k 0x00000040		/* These two used only by ChanServ */
#define CMODE_l 0x00000080

#define CMODE_R 0x00000100		/* Only identified users can join */
#define CMODE_r 0x00000200		/* Set for all registered channels */

/* This mode is for IRC_BAHAMUT servers only. */
#ifdef IRC_BAHAMUT
#define CMODE_c 0x00000400		/* Colors can't be used */
#define CMODE_M 0x00000800      /* Non-regged nicks can't send messages */
#define CMODE_j 0x00001000      /* Join rate limit */
#define CMODE_O 0x00008000		/* Only opers can join */
#endif

/* These modes are for IRC_ULTIMATE servers only. */
#ifdef IRC_ULTIMATE
#define CMODE_f 0x00000400
#define CMODE_x 0x00000800
#define CMODE_A 0x00001000
#define CMODE_I 0x00002000
#define CMODE_K 0x00004000
#define CMODE_L 0x00008000
#define CMODE_O 0x00010000
#define CMODE_S 0x00020000
#endif

/* These modes are for IRC_UNREAL servers only. */
#ifdef IRC_UNREAL
#define CMODE_c 0x00000400
#define CMODE_A 0x00000800
#define CMODE_j 0x00001000
#define CMODE_K 0x00002000
#define CMODE_L 0x00004000
#define CMODE_O 0x00008000		
#define CMODE_Q 0x00010000
#define CMODE_S 0x00020000
#define CMODE_V 0x00040000
#define CMODE_f 0x00080000
#define CMODE_G 0x00100000
#define CMODE_C 0x00200000
#define CMODE_u 0x00400000
#define CMODE_z 0x00800000
#define CMODE_N 0x01000000
#define CMODE_M 0x02000000
#define CMODE_T 0x04000000
#endif

/*************************************************************************/

/* Ignorance list data. */

typedef struct ignore_data {
    struct ignore_data *next;
    char who[NICKMAX];
    time_t time;	/* When do we stop ignoring them? */
} IgnoreData;

/*************************************************************************/

/* Languages.  Never insert anything in the middle of this list, or
 * everybody will start getting the wrong language!  If you want to change
 * the order the languages are displayed in for NickServ HELP SET LANGUAGE,
 * do it in language.c.
 */
#define LANG_EN_US              0	/* United States English */
#define LANG_JA_JIS             1	/* Japanese (JIS encoding) */
#define LANG_JA_EUC             2	/* Japanese (EUC encoding) */
#define LANG_JA_SJIS            3	/* Japanese (SJIS encoding) */
#define LANG_ES                 4	/* Spanish */
#define LANG_PT                 5	/* Portugese */
#define LANG_FR                 6	/* French */
#define LANG_TR                 7	/* Turkish */
#define LANG_IT                 8	/* Italian */
#define LANG_DE                 9   /* German */
#define LANG_CAT               10   /* Catalan */
#define LANG_GR                11   /* Greek */

#define NUM_LANGS              12	/* Number of languages */
#define USED_LANGS              9	/* Number of languages provided */

#define DEF_LANGUAGE	LANG_EN_US

/*************************************************************************/

#include "extern.h"

/*************************************************************************/

#endif	/* SERVICES_H */
